##Much of this code written by Jin Woo Jang, Rachel Vishnepolsky, Xuran Wang and Tom Peters
# Documentation
class ChainCx(DiGraph):
    """
    A class for chain complexes over Z/2.

    Extends DiGraph.

    EXAMPLES:
    sage: egCx=ChainCx({0:[1,2], 1:[3], 2:[3], 3:[]})
    sage: egCx.is_complex()
    True
    sage: egCx.homology()
    0
    
    Fails if d^2 is not zero:

    sage: ChainCx({0:[1,2], 1:[], 2:[3], 3:[]})
    Traceback (click to the left for traceback)
    ...
    Exception: Does not define a chain complex -- d squared not zero.
    """

    def __init__(self, data=None, pos=None, loops=None, format=None, boundary=[], weighted=None, implementation='networkx', sparse=True, vertex_labels=True, **kwds):
        DiGraph.__init__(self, data, pos, loops, format, boundary, weighted, implementation, sparse, vertex_labels, **kwds)
        if self.is_complex() != True:
#            print "d squared not zero."
            raise Exception("Does not define a chain complex -- d squared not zero.")

    def grandchildren(self,v):
        "Compiles a list of grandchildren of a vertex v"
        list=[]
        for w in self.neighbors_out(v):
            for y in self.neighbors_out(w):
                list.append(y)
        return list    	    


    def is_complex(self):
        """
        Checks if the graph is a complex
        """
        for v in self.vertices():
            if v in self.grandchildren(v):
                return False
            else:
                for w in self.grandchildren(v):
                    L = self.all_paths(v,w)
                    if len(L)%2!=0:
                        return False
        return True

    def homology(self):
        """
        Simplifies the graph by isolating an edge, directed from the vertex 'tail' to the vertex 'head:'
        Returns the dimension of the homology.
        """
    # Maintains a dynamic list of edges:
        edges=self.edges()
        while len(edges)>0:
            e=edges[0]
    # Indicates the starting and ending points of the edge we wish to isolate:
            head=e[1]
            tail=e[0]    
            
# Now we make a list of all the other children of the tail, and delete them. 
            sib=self.neighbors_out(tail)
            
            sib.remove(head)

            for y in sib:
                self.delete_edge(tail,y)
                
                edges.remove((e[0], y, None))

# Then we make a list of all the other parents of the head, and delete the edges from them to the head.
            par=self.neighbors_in(head)
            par.remove(tail)

            for y in par:
                self.delete_edge(y,head)
                edges.remove((y, e[1], None))
# Next we delete the edges from the grandparents to the tail.
            gp=self.neighbors_in(tail)
            for y in gp:
                self.delete_edge(y,tail)
                edges.remove((y, e[0], None))
# Now we delete the edges from the head to the grandchildren. 
            gc=self.neighbors_out(head)
            for y in gc:
                self.delete_edge(head,y)
                edges.remove((e[1], y, None))
# Now we either delete or add an edge depending on whether there is an edge between the other children and the other parents.
            for x in par:
                for y in sib:
                    if y in self.neighbors_out(x):
                        self.delete_edge(x,y)
                        edges.remove((x, y, None))
                    else: 
                        self.add_edge(x,y)
                        edges.append((x, y, None))
#FINALLY... we delete the edge between the tail and the head!
            self.delete_vertices([head,tail])
            edges.remove((e[0], e[1], None))
        return self.connected_components_number()
#        print 'Homology is ',self.connected_components_number(),' dimensional'

    def __repr__(self):
        return "Chain complex with "+repr(len(self.vertices()))+" generators."

    def tensor(self, other):
        """
        Returns a graph representing the tensor product of the two complexes self and other
        """
        result = ChainCx()
        for v in self.vertices():
            for w in other.vertices():
                result.add_vertex((v,w))
        for v in result.vertices():
            for w in self.neighbors_out(v[0]):
                result.add_edge( (v,(w,v[1])) )
            for w in other.neighbors_out(v[1]):
                result.add_edge( (v,(v[0],w)) )
        return result



#Different in this version: algebra elements now have a parameter "spinc", the number of strands - g.

#Still to implement:
#Strand_Diagram: small_grading
#SmallGradingGroup: everything.
#Document AlgElt

class PMC():
    """A pointed matched circle.

    Inputs:
    genus: the genus of the underlying surface. Number of points is 4*genus.
    matching: A list of pairs (two tuples) of elements of range(4*genus).

    Data:
    genus: same as input 'genus.'
    matching: same as input 'matching', but stored as a tuple.
    matching_inv: a function which takes points of the pmc to their matched pairs.
    basis_computed: True if we have already computed a basis for this PMC, false otherwise.
    basis[i]: the basis for the algebras associated to this PMC, in idempotent i. Is *not* computed when pmc is initialized, since it's slow and takes lots of memory.
    trunc_basis_computed: True if we have already computed a truncated basis for this PMC, false otherwise.
    trunc_basis: those elements of basis with multiplicity at most 1 everywhere.
    """

    def __init__(self,genus,matching):
        self.genus=genus
        #Sanitize the matching, so that each pair is smaller number first.
        better_matching = list()
        for (i,j) in matching:
            if i<j:
                better_matching.append((i,j))
            if j<i:
                better_matching.append((j,i))
            if i==j:
                raise Exception("Can't have a point matched to itself!")
        better_matching.sort()
        self.matching=tuple(better_matching)
        self.basis_computed = dict()
        for i in range(-genus,genus+1):
            self.basis_computed[i]=False
        self.basis = dict()
        self.trunc_basis_computed = dict()
        for i in range(-genus,genus+1):
            self.trunc_basis_computed[i]=False
        self.trunc_basis = dict()
        self.matching_inv=dict()
        for s in self.matching:
            self.matching_inv[s[0]]=s[1]
            self.matching_inv[s[1]]=s[0]

    def __repr__(self):
        return 'Genus '+repr(self.genus)+' pointed matched circle with matching '+repr(self.matching)

    def __eq__(self,other):
        #Slightly wrong: order in each pair of matching shouldn't matter, either.
        if (other.genus==self.genus) and (Set(other.matching) == Set(self.matching)):
            return True
        return False
    
    def __ne__(self,other):
        if self.__eq__(other):
            return False
        return True

    def __hash__(self):
        return hash(self.matching)

    def selftest(self):
        "Test whether given data is, in fact, a pointed matched circle."
        #Check if length is right.
        if len(self.matching) != 2*self.genus:
            raise Exception("Matching not the right length.")
        #Check each strand appears exactly once in matching.
        points_list = range(4*self.genus)
        for m in self.matching:
            for i in m:
                if i not in points_list:
                    raise Exception("Point "+i+"appears twice in matching, or is outside matching range.")
                points_list.remove(i)
        #Check matching leads to a connected surface. - WRITE
        return True

    def which_pair(self, arc):
        "Return which pair (element of self.matching) the arc belongs to."
        for m in self.matching:
            if arc in m:
                return m
        raise Exception("No idempotent seems to contain arc.")

    def matched_point(self, point):
        "Return the point matched with the given one."
        pair = self.which_pair(point)
        if point == pair[0]:
            return pair[1]
        return pair[0]

    def idempotents(self, spinc=0):
        """Returns a list of the idempotents for this pointed matched circle with g+i strands.
        Idempotents are given as lists of elements of matching."""
        return Combinations(self.matching,self.genus+spinc).list()

    def compute_basis(self, spinc=0):
        "Compues a basis for Alg(PMC, i=spinc). Stores in self.basis[spinc]"
        idems=self.idempotents(spinc)
        self.basis[spinc] = list()
        for [i,j] in CartesianProduct(idems,idems).list():
            ijcompat = list()
            ijcompat.append([i,j,[]])
            for t in range(len(i)):
                ijcompatnew=list()
                for x in ijcompat:
                    for termPair in x[1]:
                        for termPoint in termPair:
                            if x[0][0][0]<termPoint:
                                newi = list(x[0])
                                newi.remove(x[0][0])
                                newj = list(x[1])
                                newj.remove(termPair)
                                newstrands = list(x[2])
                                newstrands.append((x[0][0][0],termPoint))
                                ijcompatnew.append([newi, newj, newstrands])
                            if x[0][0][0]==termPoint:
                                newi = list(x[0])
                                newi.remove(x[0][0])
                                newj = list(x[1])
                                newj.remove(termPair)
                                newstrands = list(x[2])
                                ijcompatnew.append([newi, newj, newstrands])
                            if x[0][0][1]<termPoint:
                                newi = list(x[0])
                                newi.remove(x[0][0])
                                newj = list(x[1])
                                newj.remove(termPair)
                                newstrands = list(x[2])
                                newstrands.append((x[0][0][1],termPoint))
                                ijcompatnew.append([newi, newj, newstrands])
                    ijcompat = ijcompatnew
                ijcompat=list(ijcompatnew)
            for x in ijcompat:
                self.basis[spinc].append(Strand_Diagram(self, x[2], left_idem=i, right_idem=j))                  
        self.basis_computed[spinc] = True

    def alg_basis(self, spinc=0):
        "Returns a basis for Alg(PMC)."
        if self.basis_computed[spinc] == False:
            self.compute_basis(spinc)
        return list(self.basis[spinc])

    def compute_trunc_basis(self, spinc=0):
        "Computes a basis for the truncated algebra (no multiplicities > 1). Stores in self.trunc_basis. Currently very inefficient."
        if self.basis_computed[spinc] == False:
            self.compute_basis(spinc)
        self.trunc_basis=list()
        for x in self.basis[spinc]:
            if x.is_in_trunc():
                self.trunc_basis[spinc].append(x)
        self.trunc_basis_computed[spinc] = True

    def alg_trunc_basis(self,spinc=0):
        "Returns a basis for the truncated algebra (no multiplicities > 1)."
        if self.trunc_basis_computed[spinc] == False:
            self.compute_trunc_basis(spinc)
        return list(self.trunc_basis[spinc])

    def opposite(self):
        "Returns the PMC obtained by orientation-reversing self."
        revmatch = list()
        for (i,j) in self.matching:
            revmatch.append((4*self.genus-j-1,4*self.genus-i-1))
        return PMC(self.genus, revmatch)

    def complementary_idem(self, idem):
        "Returns the idempotent in self.opposite() complementary to i."
        answer = list()
        for (i,j) in self.matching:
            if not (i,j) in idem:
                answer.append((4*self.genus-j-1,4*self.genus-i-1))
        return answer

    def propagated_left_equals_right(self, strands, left_idem, right_idem):
        ri = list(left_idem)
        for rho in strands:
            if self.which_pair(rho[0]) not in ri:
                raise Exception("Unable to propagate left idempotent: missing element.")
            ri.remove(self.which_pair(rho[0]))
        for rho in strands:
            if self.which_pair(rho[1]) in ri:
                raise Exception("Unable to propagate left idempotent: repeated element.")
            ri.append(self.which_pair(rho[1]))
        if (right_idem !=[]) and (right_idem != ri):
            return False
        return True

    def show(self):
        circ = line([(0,-.5),(0,4*self.genus-.5)], rgbcolor=(0,0,0))
        pict = circ
        for i in range(4*self.genus):
            pict = pict + line([(-.25,i),(.25,i)], rgbcolor=(.1,.1,.1))
        for pair in self.matching:
            j=self.matching.index(pair)
            offset = j/(2*self.genus)
            pict = pict+line([(-.3,pair[0]),(-.5-offset,pair[0]),(-.5-offset,pair[1]),(-.3,pair[1])], rgbcolor=(.3-j/(5*self.genus),.6+j/(5*self.genus),.3-j/(5*self.genus)))
        pict.show(axes=False, xmin=-2,xmax=2, ymin=-.5, ymax=4*self.genus-.5)

    def zero(self):
        "Return the zero element of self."
        return AlgElt([],self)

    def chords(self, spinc=0):
        "Return sum_(xi a chord)a(xi), in specified strands grading. Defaults to middle strands grading."
        arc_list = list()
        for i in range(4*self.genus):
            for j in range(i+1,4*self.genus):
                arc_list.append((i,j))
        answer = self.zero()
        for xi in arc_list:
            answer = answer + alg_element(self,[xi],spinc)
        return answer

    def arcslide(self, i, j):
        "Return the PMC obtained from self by doing an arcslide of arc end i over arc ending in j.  i and j must be consecutive."
        if abs(i-j) != 1:
            raise Exception("Can only slide a handle over a consecutive one.")
        answer_matching = list()
        jp = self.matched_point(j)
        if i<jp:
            for (a,b) in self.matching:
                if (a < i) or (a>jp):
                    newa = a
                if i<a<jp:
                    newa = a-1
                if (b < i) or (b>jp):
                    newb = b
                if i<b<jp:
                    newb = b-1
                if self.is_overslide(i,j):
                    if a == i:
                        newa = jp
                    if a == jp:
                        newa = jp-1
                    if b == i:
                        newb = jp
                    if b == jp:
                        newb = jp-1
                if self.is_underslide(i,j):
                    if a == i:
                        newa = jp-1
                    if a == jp:
                        newa = jp
                    if b == i:
                        newb = jp-1
                    if b == jp:
                        newb = jp                
                answer_matching.append((newa,newb))
        if i>jp:
            for (a,b) in self.matching:
                if (a > i) or (a<jp):
                    newa = a
                if jp<a<i:
                    newa = a+1
                if (b > i) or (b<jp):
                    newb = b
                if jp<b<i:
                    newb = b+1
                if self.is_overslide(i,j):
                    if a == i:
                        newa = jp
                    if a == jp:
                        newa = jp+1
                    if b == i:
                        newb = jp
                    if b == jp:
                        newb = jp+1
                if self.is_underslide(i,j):
                    if a == i:
                        newa = jp+1
                    if a == jp:
                        newa = jp
                    if b == i:
                        newb = jp+1
                    if b == jp:
                        newb = jp
                answer_matching.append((newa,newb))
        return PMC(self.genus, answer_matching)

    def is_underslide(self, i, j):
        "Returns True if sliding i over j is an overslide, and False if it's an underslide."
        jp = self.matched_point(j)
        if ((jp < i) and (i<j)) or ( (j<i) and (i<jp)):
            return True
        return False

    def is_overslide(self, i, j):
        "Returns True if sliding i over j is an overslide, and False if it's an underslide."
        jp = self.matched_point(j)
        if ((jp < i) and (j<i)) or ( (i<jp) and (i<j)):
            return True
        return False

class Strand_Diagram():
    """A strand diagram for a pointed matched circle.

    Elements of the algebra are to be stored as lists of Strand_Diagram's.

    Inputs:
    pmc: the pointed matched circle in which this is a strand diagram.
    strands: and list of pairs in points in the pmc (range(4*genus)).
    left_idem: the left idempotent of the diagram. Leave blank to have it computed from the right idempotent.
    right_idem: the right idempotent of the diagram. Leave blank to have it computed from the left idempotent.
    name: optionally, a name for this strand diagram; if given, the name is printed; if not, the key data is printed.
    spinc: the spinc structure. Number of elements of idempotent is pmc.genus + spinc. Defaults to 0.

    Example:
    my_pmc = split_matching(1)
    my_idemp = my_pmc.which_pair(1)
    my_chord = Strand_Diagram(my_pmc, [(1,2)], left_idem=[my_idemp])
    """
    def __init__(self, pmc, strands, left_idem=[], right_idem=[], name=''):
        self.pmc=pmc; self.strands=strands; 
        #Propogate idempotents.
        if left_idem != []:
            self.name = name
            self.left_idem=left_idem
            ri = list(left_idem)
            for rho in strands:
                if self.pmc.which_pair(rho[0]) not in ri:
                    raise Exception("Unable to propagate left idempotent: missing element.")
                ri.remove(self.pmc.which_pair(rho[0]))
            for rho in strands:
                if self.pmc.which_pair(rho[1]) in ri:
                    raise Exception("Unable to propagate left idempotent: repeated element.")
                ri.append(self.pmc.which_pair(rho[1]))
            if (right_idem !=[]) and (Set(right_idem) != Set(ri)):
                print "Propogated: ",ri,"  Given: ",right_idem
                raise Exception("Propogated right idempotent, does not agree with given right idempotent.")
            self.right_idem=ri
        else: 
            self.name = name
            self.right_idem=right_idem
            li = list(right_idem)
            reverse_strands=list(strands)
            reverse_strands.reverse()
            for rho in reverse_strands:
                if self.pmc.which_pair(rho[1]) not in li:
                    raise Exception("Unable to propagate right idempotent: missing element.")
                li.remove(self.pmc.which_pair(rho[1]))
                if self.pmc.which_pair(rho[0]) in li:
                    raise Exception("Unable to propagate right idempotent: repeated element.")
                li.append(self.pmc.which_pair(rho[0]))
            self.left_idem=li
        self.strands.sort()
        self.left_idem.sort()
        self.right_idem.sort()
        self.spinc=len(self.left_idem)
            
    def __repr__(self):
        if self.name:
            return self.name
        return ' | LI:'+repr(self.left_idem)+' S:'+repr(self.strands)+' RI:'+repr(self.right_idem)+' | '

    def __eq__(self, other):
        if not isinstance(other, Strand_Diagram):
            return False
        if (self.pmc==other.pmc) and (Set(self.strands)==Set(other.strands)) and (Set(self.left_idem)==Set(other.left_idem)):
            return True
        return False
    
    def __ne__(self,other):
        if self.__eq__(other):
            return False
        return True

    #This seems to work, but RL a little concerned this goes beyond his coding skills.
    def __hash__(self):
        return hash(tuple(self.strands)).__xor__(hash(tuple(self.left_idem)))

    def __mul__(self, other):
        return self.r_multiply(other)
    
    def name_me(self, name):
        self.name=name

    def multiplicities(self):
        "Returns the list of local multiplicities of self."
        local_mults = (4*self.pmc.genus-1)*[0]
        for rho in self.strands:
            for i in range(rho[0],rho[1]):
                local_mults[i]+=1
        return local_mults

    def is_in_trunc(self):
        "Returns True if none of the local multiplicities of self are >1 and False otherwise."
        local_mults = self.multiplicities()
        for i in range(4*self.pmc.genus-1):
            if local_mults[i]>1:
                return False
        return True

    def inv(self):
        "Returns the number of inversions of self. Used by big_grading()."
        answer=0
        for rho in self.strands:
            for sigma in self.strands[self.strands.index(rho):]:
                if rho[0]<sigma[0] and rho[1]>sigma[1]:
                    answer+=1
        return answer

    def iota(self):
        "Returns iota(self), as in LOT Definition 3.18. Used by big_grading."
        local_mults = self.multiplicities()
        m = 0/1
        reduced_starting_idem = list()
        for s in self.strands:
            reduced_starting_idem.append(s[0])
        for i in reduced_starting_idem:
            m = m + local_mults[i]/2
            if i != 0:
                m = m + local_mults[i-1]/2
        return self.inv()-m

    def big_grading(self):
        "Returns G' (big grading group) grading of self."
        answer = list()
        answer.append(self.iota())
        answer.extend(self.multiplicities())
        return answer

    def small_grading(self):
        "Returns G (small grading group) grading of self."
        pass

    #RL August 5 2011: fixed bug: differential() used to not check for double crossings. (Thanks, Bohua Zhan.)
    def differential(self):
        "Return the differential of self."
        answer = list()
        grgrp = BigGradingGroup(self.pmc)
        #First run over all of the moving strands.
        for x in self.strands:
            for y in self.strands[self.strands.index(x):]:
                if (x[0]<y[0]) and (x[1]>y[1]):
                    new_strands = list(self.strands)
                    new_strands[self.strands.index(x)]=(x[0],y[1])
                    new_strands[self.strands.index(y)]=(y[0],x[1])
                    append_me = Strand_Diagram(self.pmc, new_strands,left_idem=self.left_idem)
                    #Test whether this element had a double crossing.
                    if grgrp.multiply(append_me.big_grading(),grgrp.central()) == self.big_grading():
                        answer.append(append_me)
        #Now find the unmoving strands:
        moving_pts = list()
        for s in self.strands:
            moving_pts.append(s[0])
        unmoving_idem=list(self.left_idem)
        for i in list(unmoving_idem):
            for p in i:
                if (p in moving_pts) and (i in unmoving_idem):
                    unmoving_idem.remove(i)
        #And consider their contribution to differential
        for i in unmoving_idem:
            for x in self.strands:
                for p in i:
                    if (x[0]<p) and (p<x[1]):
                        new_strands=list(self.strands)
                        new_strands.remove(x)
                        new_strands.append((p,x[1]))
                        new_strands.append((x[0],p))
                        append_me = Strand_Diagram(self.pmc,new_strands,left_idem=self.left_idem)
                    #Test whether this element had a double crossing.
                        if grgrp.multiply(append_me.big_grading(),grgrp.central()) == self.big_grading():
                            answer.append(append_me)
        return answer

    def r_multiply(self, other):
        "Return the product self times other, where other is a Strand_Diagram."
        #Do the idempotents agree?
        if self.right_idem != other.left_idem:
            return []
        right_endpoints = list()
        for s in self.strands:
            right_endpoints.append(s[1])
        unmoving_arcs = list()
        for x in self.right_idem:
            if (not (x[0] in right_endpoints)) and (not (x[1] in right_endpoints)):
                unmoving_arcs.append(x)
        unmoving_points = list()
        for x in unmoving_arcs:
            unmoving_points.append(x[0])
            unmoving_points.append(x[1])
        #Do the idempotents agree but the strands not match up?
        for s in other.strands:
            if (not (s[0] in right_endpoints)) and (not (s[0] in unmoving_points)):
                return []
        #Join the strands that can be joined; take union of rest.
        answer_strands = list(self.strands)
        for s in other.strands:
            if s[0] in right_endpoints:
                for x in self.strands:
                    if x[1]==s[0]:
                        answer_strands.remove(x)
                        answer_strands.append((x[0],s[1]))
            else:
                answer_strands.append(s)
        #Now check: were there double crossings?
        answer=Strand_Diagram(self.pmc, answer_strands, self.left_idem)
        grgrp = BigGradingGroup(self.pmc)
        if grgrp.multiply(self.big_grading(),other.big_grading()) != answer.big_grading():
            return []
        return answer
    
    def l_multiply(self, other):
        "Return the product other times self."
        return other.r_multiply(self)

    def augmentation(self):
        "Returns augmentation(self), i.e., self if self is an idempotent, empty list otherwise."
        if self.strands == []:
            return self
        return []

    def id_minus_aug(self):
        "Returns (Id - augmentation)(self), i.e., self if self is not an idempotent, empty list otherwise."
        if self.strands != []:
            return self
        return []

    def show(self):
        #First draw the pointed matched circle.
        circ = line([(0,-.5),(0,4*self.pmc.genus-.5)], rgbcolor=(0,0,0))
        pict = circ
        for i in range(4*self.pmc.genus):
            pict = pict + line([(-.25,i),(.25,i)], rgbcolor=(.1,.1,.1))
        for pair in self.pmc.matching:
            j=self.pmc.matching.index(pair)
            offset = j/(2*self.pmc.genus)
            pict = pict+line([(-.3,pair[0]),(-.5-offset,pair[0]),(-.5-offset,pair[1]),(-.3,pair[1])], rgbcolor=(.3-j/(5*self.pmc.genus),.6+j/(5*self.pmc.genus),.3-j/(5*self.pmc.genus)))
        #Now draw the moving strands
        for s in self.strands:
            i=self.strands.index(s)
            pict = pict+line([(.3,s[0]),(1.7,s[1])],rgbcolor=(1-i/(6*self.pmc.genus),.1+i/(6*self.pmc.genus),.1+i/(6*self.pmc.genus)))
        #Find unmoving strands
        moving_pts = list()
        for s in self.strands:
            moving_pts.append(s[0])
        unmoving_idem=list(self.left_idem)
        for i in list(unmoving_idem):
            for p in i:
                if (p in moving_pts) and (i in unmoving_idem):
                    unmoving_idem.remove(i)
        #Now draw the non-moving strands
        for i in unmoving_idem:
            j=unmoving_idem.index(i)
            pict = pict+line([(.3,i[0]), (1.7,i[0])], linestyle='--', rgbcolor=(.1+j/(6*self.pmc.genus),.1+j/(6*self.pmc.genus),1-j/(6*self.pmc.genus)))
            pict = pict+line([(.3,i[1]), (1.7,i[1])], linestyle='--', rgbcolor=(.1+j/(6*self.pmc.genus),.1+j/(6*self.pmc.genus),1-j/(6*self.pmc.genus)))
        #Also draw circle on RHS
        pict = pict+line([(2,-.5),(2,4*self.pmc.genus-.5)], rgbcolor=(0,0,0))
        for i in range(4*self.pmc.genus):
            pict = pict + line([(1.75,i),(2.25,i)], rgbcolor=(.1,.1,.1))
        for pair in self.pmc.matching:
            j=self.pmc.matching.index(pair)
            offset = j/(2*self.pmc.genus)
            pict = pict+line([(2.3,pair[0]),(2.5+offset,pair[0]),(2.5+offset,pair[1]),(2.3,pair[1])], rgbcolor=(.3-j/(5*self.pmc.genus),.6+j/(5*self.pmc.genus),.3-j/(5*self.pmc.genus)))
        pict.show(axes=False)

    def opposite(self):
        "Returns the Strand_Diagram with the same multiplicities and idempotents as self but viewed as in opposite PMC."
        rev_right_idem = list()
        for (i,j) in self.left_idem:
            rev_right_idem.append((4*self.pmc.genus-1-j,4*self.pmc.genus-1-i))
        rev_left_idem = list()
        for (i,j) in self.right_idem:
            rev_left_idem.append((4*self.pmc.genus-1-j,4*self.pmc.genus-1-i))
        rev_strands = list()
        for (i,j) in self.strands:
            rev_strands.append((4*self.pmc.genus-1-j,4*self.pmc.genus-1-i))
        return Strand_Diagram(self.pmc.opposite(),rev_strands,rev_left_idem,rev_right_idem)

from UserList import UserList
class AlgElt(UserList):
    """An element of Alg(PMC). Roughly, a list of Strand_Diagram's.
    Inputs:
    data: a list or tuple of Strand_Diagram's. Should all be over the same PMC.
    pmc (optional): pointed matched cirlce underlying this AlgElt. If data != [] then this is irrelevant. If data==[] then it's good (but not forced) to specify pmc.

    Alternately, data can be a single Strand_Diagram. In this case, pmc argument is ignored.

    Methods: 

    Examples:
    """
    def __init__(self, data, pmc=[]):
        if isinstance(data, Strand_Diagram):
            self.pmc = data.pmc
            UserList.__init__(self, [data])
        else:
            UserList.__init__(self, data)
            if pmc:
                self.pmc=pmc
            else:
                if data:
                    self.pmc=data[0].pmc
            if data:
                for x in data:
                    if self.pmc != x.pmc:
                        raise Exception("All elements in AlgElt must come from same PMC.")
            self.reduce_mod_2()

    def __repr__(self):
        return "<"+repr(self.data)+">"

    def __mul__(self, other):
        if not isinstance(other, AlgElt):
            return other.__rmul__(self)
        answer = list()
        for x in self:
            for y in other:
                if x.r_multiply(y):
                    answer.append(x.r_multiply(y))
        return AlgElt(answer)

    def __add__(self, other):
        answer = self.data + other.data
        return AlgElt(reduce_mod_2(answer))

    def __eq__(self, other):
        #For convenience, the element 0 is also equal to the empty list.
        if not self.data:
            if not other:
                return True
            return False
        if not isinstance(other, AlgElt):
            return False
        if Set(self.data)==Set(other.data):
            return True
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    #Probably works... though mutable objects shouldn't be hashable...
    def __hash__(self):
        return Set(self.data).__hash__()

    def __pow__(self, other):
        "a**b now gives a\otimes b, an AlgBlgElt."
        if not isinstance(other, AlgElt):
            raise Exception("What used to be the power operation takes two algebra elements and gives their external tensor product, and AlgBlgElt.")
        answer = AlgBlgElt({},self.pmc, other.pmc)
        for x in self.data:
            answer = answer + AlgBlgElt({x:other})
        return answer

    def diff(self):
        "Returns differential(self)."
        answer = list()
        for x in self:
            answer.extend(x.differential())
        return AlgElt(answer, self.pmc)

    def differential(self):
        "Returns differential(self)."
        return self.diff()

    def reduce_mod_2(self):
        "Deletes pairs of duplicates. Does not try to preserve order."
        answer = list()
        for x in self.data:
            if x in answer:
                answer.remove(x)
            else:
                answer.append(x)
        self.data = answer

    def pmc(self):
        if not self.data:
            return False
    def opposite(self):
        "Returns the AlgElt obtained by taking opposite of all Strand_Diagram's in self."
        rev_data = list()
        for x in self.data:
            rev_data.append(x.opposite())
        return AlgElt(rev_data, self.pmc.opposite())

    def is_invertible(self):
        "Returns True is self is an idempotent and False otherwise. (SHOULD REVISE.)"
        if len(self.data)==0:
            return False
        for x in self.data:
            if x != x*x:
                return False
        return True
    
    def inverse(self):
        "Currently only works for idempotents."
        if not self.is_invertible():
            raise Exception("Can not invert non-invertible element.")
        return AlgElt(self.data, self.pmc)

#Some particular pmc's.
def split_matching(g):
    match = list()
    for i in range(g):
        match.append((4*i,4*i+2))
        match.append((4*i+1,4*i+3))
    return PMC(g,match)

def antipodal_matching(g):
    match = list()
    for i in range(2*g):
        match.append((i,i+2*g))
    return PMC(g,match)


#Some useful operations for algebras associated to pmc's.
def alg_element(pmc, strands, spinc=0):
    "Returns list (sum) of all algebra elements in A(pmc) with the given strands, and any consistent idempotents. Naively written at present."
    g = pmc.genus+spinc
    answer = list()
    if not isinstance(strands, list):
        raise Exception("Strands input to alg_element should be a list.")
    for i in pmc.idempotents(spinc):
        if l_idem_compat(pmc,strands,i):
            answer.append(Strand_Diagram(pmc,strands,left_idem=i))

    return AlgElt(answer, pmc) #Used to just return answer.

def l_idem_compat(pmc, strands, idem):
    "Check if strands are compatible with the left idempotent idem of pmc."
#    if idem not in pmc.idempotents():
#        raise Exception("Given idmpotent idem is not an idempotent for given matched circle pmc.")
    delete_list = list(idem)
    add_list = list()
    for s in strands:
        if (pmc.which_pair(s[0]) not in delete_list) or (pmc.which_pair(s[1]) in add_list):
            return False
        delete_list.remove(pmc.which_pair(s[0]))
        add_list.append(pmc.which_pair(s[1]))
    for i in delete_list:
        if i in add_list:
            return False
    return True

def r_idem_compat(pmc, strands, idem):
    "Check if strands are compatible with the right idempotent idem of pmc."
#    if idem not in pmc.idempotents():
#        raise Exception("Given idmpotent idem is not an idempotent for given matched circle pmc.")
    delete_list = list(idem)
    add_list = list()
    for s in strands:
        if (pmc.which_pair(s[1]) not in delete_list) or (pmc.which_pair(s[0]) in add_list):
            return False
        delete_list.remove(pmc.which_pair(s[1]))
        add_list.append(pmc.which_pair(s[0]))
    for i in delete_list:
        if i in add_list:
            return False
    return True


def mul_list(list1, list2):
    "Multiply two lists (sums) of Strand_Diagrams."
    answer = list()
    for x in list1:
        for y in list2:
            if x.r_multiply(y):
                answer.append(x.r_multiply(y))
    return answer

def alg_differentiate_list(x):
    "Differentiate all the elements in the given list x."
    answer = list()
    for s in x:
        answer.extend(s.differential())
    return answer

def aug_list(x):
    "Apply the augmentation to all elements of list; return result."
    answer = list()
    for s in x:
        if s.augmentation():
            answer.append(s.augmentation())
    return answer

def id_minus_aug_list(x):
    "Apply (Id-Augmentation) to all elements of list; return result."
    answer = list()
    for s in x:
        if s.id_minus_aug():
            answer.append(s.id_minus_aug())
    return answer

#Three self-tests for the algebra, in specified spinc structure.
def alg_d_squared_zero(pmc, spinc=0):
    "Return True if d-squared is zero for Alg(pmc), and False otherwise."
    for a in pmc.alg_basis(spinc):
        if reduce_mod_2(alg_differentiate_list(a.differential())):
            return False
    return True

def alg_is_leibniz(pmc, spinc=0):
    "Return True if Alg(pmc) satisfies the Leibniz rule, and False otherwise."
    for a in pmc.alg_basis(spinc):
        for b in pmc.alg_basis(spinc):
            test_list = mul_list(a.differential(),[b])
            other_order = mul_list([a],b.differential())
            if other_order:
                test_list.extend(other_order)
            if a.r_multiply(b):
                test_list.extend((a.r_multiply(b)).differential())
            if reduce_mod_2(test_list):
                return False
    return True

def alg_is_assoc(pmc,spinc=0):
    "Return True if Alg(pmc) is associative, and False otherwise."
    #Written very badly.
    for a in pmc.alg_basis(spinc):
        for b in pmc.alg_basis(spinc):
            for c in pmc.alg_basis(spinc):
                if (b.r_multiply(c)):
                    if a.r_multiply(b):
                        if not ((a.r_multiply(b.r_multiply(c))) == ((a.r_multiply(b)).r_multiply(c))):
                            return False
                    else:
                        if a.r_multiply(b.r_multiply(c)):
                            return False
                else:
                    if a.r_multiply(b):
                        if (a.r_multiply(b)).r_multiply(c):
                            return False
    return True

def alg_homology(pmc,spinc=0):
    "Returns the dimension of the homology of A(pmc,0)"
    vert = pmc.alg_basis(spinc)
#    print vert
#    for i in list(vert):
#        if (i.left_idem != [(0,2),(1,3)]) or (i.right_idem !=[(0,2),(1,3)]):
#            vert.remove(i)
    edges = dict()
    for v in vert:
        edges[v]=v.differential()
    chcx=ChainCx(edges)
    return chcx.homology()

def reduce_mod_2(elts):
    "Takes a list and deletes pairs of duplicates. Does not try to preserve order."
    answer = list()
    for x in elts:
        if x in answer:
            answer.remove(x)
        else:
            answer.append(x)
    return answer

class BigGradingGroup():
    """The big grading group associated to a pointed matched circle.
    Elements are lists [m,a_1,...] of length 4*genus where:
    m is the homological (Maslov) component of the grading and the
    a_i are the homological components of the gradings.
    The m and a_i should be Rational's or Integer's.
    Inputs: a pointed matched circle (class PMC)"""
    
    def __init__(self, pmc):
        self.pmc = pmc

    def is_element(self, x):
        "Check if x is an element of self."
        if not isinstance(x,list):
            return False
        if len(x) != 4*self.pmc.genus:
            return False
        for t in x:
            if (not isinstance(t,sage.rings.rational.Rational)) and (not isinstance(t,sage.rings.integer.Integer)):
                return False
        return True
    
    def multiply(self, x, y):
        if (not self.is_element(x)):
            raise Exception("First input is not an element of this BigGradingGroup.")
        if (not self.is_element(y)):
            raise Exception("Second input is not an element of this BigGradingGroup.")
        z=list(x)
        for i in range(4*self.pmc.genus):
            z[i]=x[i]+y[i]
        for i in range(1,4*self.pmc.genus-1):
            z[0]=z[0]+(x[i]*y[i+1]-x[i+1]*y[i])/2
        return z

    def inverse(self, x):
        if (not self.is_element(x)):
            raise Exception("Input is not an element of this BigGradingGroup.")
        z=list()
        for i in range(4*self.pmc.genus):
            z.append(-x[i])
        return z


    def multiply_list(self, list_of_elts):
        "Given a list [x_1,x_2,...] of elements of G' returns the element x_1*x_2*..."
        z=self.identity()
        for x in list_of_elts:
            z=self.multiply(z,x)
        return z

    def power(self, x, n):
        "Return x^n"
        if n==0:
            return self.identity()
        if n<0:
            to_raise=self.inverse(x)
            pow = -n
        if n>0:
            to_raise=x
            pow = n
        return self.multiply_list(pow*[to_raise])

    def identity(self):
        return (4*self.pmc.genus)*[0]

    def central(self):
        "Return the distinguished central element of this BigGradingGroup"
        return [1]+(4*self.pmc.genus-1)*[0]

class SmallGradingGroup():
    """The small grading group associated to a pointed matched circle.
    Inputs: a pointed matched circle (class PMC)"""
    
    def __init__(self, pmc):
        pass
    
    def multiply(self, x, y):
        pass

    def inverse(self, x):
        pass
    
    def multiply_list(self, list_of_elts):
        pass

    def central(self):
        pass



from UserDict import UserDict
class AlgBlgElt(UserDict):
    """An element of Alg(PMC1)\otimes Alg(PMC2). 
    Stored as a dictionary with keys Strand_Diagram's for PMC1 and values AlgElt's for PMC2.

    Inputs:
    data: A dictionary with keys Strand_Diagram's for PMC1 and values AlgElts for PMC2.
    pmc_1 (optional): first pointed matched cirlce underlying this AlgElt. If data != [] then this is irrelevant. If data==[] then it's good (but not forced) to specify pmc.
    pmc_2 (optional): second pointed matched cirlce underlying this AlgElt. If data != [] then this is irrelevant. If data==[] then it's good (but not forced) to specify pmc.

    Methods: 

    Examples:
    sage:  ab = AlgBlgElt({rho1:AlgElt([rho1,rho3]),rho2:AlgElt([rho2])},small_pmc,small_pmc)

    sage:  myelt = AlgBlgElt({rho1:AlgElt([rho1,rho2,rho3]),rho2:AlgElt([rho1,rho23])})
    sage:  myelt.basis_expansion()
    [(rho1, rho2), (rho1, rho3), (rho2, rho1), (rho2, rho23)]
    sage:  myelt*myelt
    {LI:[(0, 2)] S:[(0, 2)] RI:[(0, 2)]: [LI:[(0, 2)] S:[(0, 3)] RI:[(1, 3)]]}
    """
    def __init__(self, data, pmc_1=[], pmc_2=[]):
        UserDict.__init__(self, data)
        if pmc_1:
            self.pmc_1=pmc_1
        else:
            if data:
                self.pmc_1=data.keys()[0].pmc
        if pmc_2:
            self.pmc_2=pmc_2
        else:
            if data:
                self.pmc_2=data.values()[0].pmc
        self.reduce_mod_2()
        #Should do more sanity checking.

    def __add__(self, other):
        if (self.pmc_1 != other.pmc_1):
            raise Exception("Can't add elements with different pmc_1's.")
        if (self.pmc_2 != other.pmc_2):
            raise Exception("Can't add elements with different pmc_2's.")
        answer = dict()
        for x in self.data.keys():
            if x in other.data.keys():
                answer[x]=self.data[x]+other.data[x]
            else:
                answer[x]=self.data[x]
        for x in other.data.keys():
            if not (x in self.data.keys()):
                answer[x]=other.data[x]
        return AlgBlgElt(answer, self.pmc_1, self.pmc_2)

    def __mul__(self, other):
        if not (isinstance(other,AlgBlgElt)):
            return other.__rmul__(self)
        if (self.pmc_1 != other.pmc_1):
            raise Exception("Can't multiply elements with different pmc_1's.")
        if (self.pmc_2 != other.pmc_2):
            raise Exception("Can't multiply elements with different pmc_2's.")
        answer = dict()
        for x in self.data.keys():
            for y in other.data.keys():
                if x*y:
                    answer[x*y]=AlgElt([],self.pmc_2)
        for x in self.data.keys():
            for y in other.data.keys():
                if x*y:
                    answer[x*y]=answer[x*y]+self.data[x]*other.data[y]
        return AlgBlgElt(answer, self.pmc_1,self.pmc_2)

    def __eq__(self, other):
        #For convenience, the element 0 is also equal to the empty list.
        if not self.data:
            if not other:
                return True
            return False
        if not isinstance(other, AlgBlgElt):
            return False
        for x in self.data.keys():
            if not (x in other.data.keys()):
                if self.data[x]:
                    return False
            else:
                if self.data[x]!=other.data[x]:
                    return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

#    #Probably works... though mutable objects shouldn't be hashable...
#    def __hash__(self):
#        return Set(self.data).__hash__()

    def diff_1(self):
        "Differentiate part of self in PMC_1; returns result."
        #A bit slow: no need to initialize AlgBlgElt's each time.
        answer = AlgBlgElt({},self.pmc_1, self.pmc_2)
        for x in self.data.keys():
            for y in x.differential():
                answer = answer + AlgBlgElt({y:self.data[x]},self.pmc_1,self.pmc_2)
        return answer

    def diff_2(self):
        "Differentiate part of self in PMC_2; returns result."
        #A bit slow: no need to initialize AlgBlgElt's each time.
        answer = AlgBlgElt({},self.pmc_1, self.pmc_2)
        for x in self.data.keys():
            answer = answer + AlgBlgElt({x:self.data[x].diff()},self.pmc_1,self.pmc_2)
        return answer

    def reduce_mod_2(self):
        "Deletes pairs of duplicates."
        answer = dict()
        for x in self.data.keys():
            self.data[x].reduce_mod_2()
        for x in self.data.keys():
            if not self.data[x]:
                del self.data[x]

    def basis_expansion(self):
        "Returns a list of pairs (a_i,b_i) of strand diagrams so that sum_i a_i\otimes b_i = self"
        self.reduce_mod_2()
        answer = list()
        for x in self.data.keys():
            for y in self.data[x]:
                answer.append((x,y))
        return answer
    
    def show(self):
        "Display self graphically."
        pass

    def exchangeLR(self):
        "Returns result of exchanging pmc_1 and pmc_2, i.e., a\otimes b becomes b\otimes a."
        answer = AlgBlgElt([],self.pmc_2, self.pmc_1)
        for (a,b) in self.basis_expansion():
            answer = answer + (AlgElt(b)**AlgElt(a))
        return answer



#Need to implement:
#simplify in class TypeDStr
#def infty_type_D_bd(k):
#def zero_type_D_bd(k):
#Improve mor_to_d
#DElt.__hash__

def paths(graph):
    "Returns a list of all oriented paths in the directed graph. Raises exception if graph has loop."
    if graph.has_loops():
        raise Exception("Graph has directed loops; set of paths not finite.")
    answer = list()
    for v in graph.vertices():
        answer.extend(paths_from_v(graph,v))
    return answer

def paths_from_v(graph,v):
    "Returns a list of all oriented paths in the directed graph starting at v. Does not check this is finie."
    answer = list()
    for w in graph.neighbors_out(v):
        for p in paths_from_v(graph,w):
            answer.append([v]+p)
    answer.append([v])
    return answer

def longest_path_len(graph):
    "Returns the length of the longest path in directed graph. Raises exception if graph has directed loop.)"
    if graph.has_loops():
        raise Exception("Graph has directed loops; set of paths not finite.")
    path_list = paths(graph)
    l=0
    for x in path_list:
        if len(x)>l:
            l=len(x)
    return l-1

class DGen():
    """A class for generators of a type D structure.

    Inputs:
    name: name for this generator.
    pmc: underlying pointed matched circle.
    idem: idempotent for this generator.

    Both pmc and idem should be given EXCEPT if just want to obtain an element with a given name.

    __eq__ JUST LOOKS AT THE NAME.  Similarly __hash__.

    Example:
    x = DGen('x',split_matching(1),[(0,2)])
    """
    def __init__(self, name, pmc=[], idem=[]):
        #Should do some sanity checking.
        self.name = name
        self.pmc = pmc
        self.idem =idem
        self.idem.sort()

    def __repr__(self):
        return self.name.__repr__()
    
    def __eq__(self, other):
        "Checks if the NAMES of self and other are the same."
        if type(self) != type(other):
            return False
        if self.name == other.name:
            return True
        return False

    def __ne__(self, other):
        return not (self.__eq__(other))

    def __hash__(self):
        return self.name.__hash__()

    def __rmul__(self, other):
        "Multiply self by alg elt other to give a DElt."
        coeff = AlgElt([Strand_Diagram(self.pmc, [], left_idem=self.idem)])
        coeff = other*coeff
        return DElt({self:coeff})

from UserDict import UserDict
class DElt(UserDict):
    """A class for elements of a left Type D structure.

    Input:
    A dict 'data' with keys x_i DGen's and values algebra elements a_i.
    Alternatively, a_i may be a list of Strand_Diagram's. Also, it's okay for x_i just to be a name, not a DGen: in that case, the idempotent will be extracted from a_i[0].
    pmc: optional argument, giving the pointed matched circle for this element. Must be given if dict is empty.

    Examples:
    small_pmc = split_matching(1)
    x = DGen('x',small_pmc,[(0,2)])
    y = DGen('y',small_pmc,[(1,3)])
    rho1=Strand_Diagram(small_pmc,[(0,1)],left_idem=[(0,2)],name='rho1')
    rho2=Strand_Diagram(small_pmc, [(1,2)],left_idem=[(1,3)],name='rho2')
    rho3=Strand_Diagram(small_pmc, [(2,3)],left_idem=[(0,2)],name='rho3')
    ax = DElt({x:AlgElt([rho2])})
    by = DElt({y:AlgElt([rho1,rho3])})
    bigelt = DElt({'x':[rho2],'y':[rho1,rho3]})
    """
    def __init__(self, data, pmc=[]):
        gooddata = dict()
        for x in data.keys():
            if isinstance(x,DGen):
                goodx = x
            else:
                goodx = DGen(x,data[x][0].pmc,data[x][0].right_idem)
            if isinstance(data[x],AlgElt):
                gooddata[goodx]=data[x]
            else:
                gooddata[goodx]=AlgElt(data[x])
        UserDict.__init__(self, gooddata)
        if pmc:
            self.pmc = pmc
        else:
            self.pmc = self.data.values()[0].pmc
        self.reduce()

#    def keys(self):
#        return self.data.keys()

    def coeff(self, x):
        return self.data[x]

    def diff_coeffs(self):
        "Returns the element obtained by differentiating the coefficients of self."
        answer = dict()
        for x in self.data.keys():
            answer[x]=list()
            for a in self.data[x]:
                answer[x].extend(a.differential())
        return DElt(answer, pmc=self.pmc)

    def basis_expansion(self):
        "Returns a list of pairs [(rho_i,x_i)], with rho_i a Strand_Diagram and x_i a DGen so that self is sum_i rho_i \otimes x_i."
        self.reduce()
        answer = list()
        for x in self.data.keys():
            for a in self.data[x]:
                answer.append((a,x))
        return answer

    def __repr__(self):
        return repr(self.data)

    def opposite(self):
        "Return result of taking opposite() of algebra elements."
        revdata = dict()
        for x in self.data.keys():
            revdata[x]=self.data[x].opposite()
        return DElt(revdata, self.pmc.opposite())

    def reduce(self):
        "Delete duplicates from self, and also keys with values the empty list."
        for x in self.data.keys():
            self.data[x].reduce_mod_2()
        for x in self.data.keys():
            if self.data[x]==[]:
                del self.data[x]

    def __add__(self, other):
        if self.pmc != other.pmc:
            raise Exception("Can't add elements over different pointed matched circles.")
        ansdict = dict()
        for x in self.data.keys():
            if not (x in other.data.keys()):
                ansdict[x]=self.data[x]
            else:
                ansdict[x] = self.data[x]+other.data[x]
        for x in other.data.keys():
            if not (x in self.data.keys()):
                ansdict[x]=other.data[x]
        return DElt(ansdict, self.pmc)
    
    def __rmul__(self, other):
        "Left multiply an element by an algebra element."
        ansdict = dict()
        for x in self.data.keys():
            ansdict[x] = other*self.data[x]
        return DElt(ansdict, self.pmc)

    def __eq__(self, other):
        if type(self) != type(other):
            return False
        if Set(self.data.keys()) != Set(other.data.keys()):
            return False
        for x in self.data.keys():
            if self.data[x] != other.data[x]:
                return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

#    def __hash__(self):
#        pass

class TypeDStr():
    """A type D structure.

    Inputs: 
    --pmc: the underlying pointed matched circle.
    --basis: a dictionary with keys names for basis elements (arbitrary) and values the corresponding idempotents in pmc 
    OR 
    a list of DGen's
    --structure_consts: a dictionary with keys the basis elements and values lists of pairs (a,x) where:
    -----a is an algebra element (or, alternatively, list of ChordDiagrams for pmc).
    -----x is a basis element.
    OR
    a dictionary with keys basis elements and values DElt's

    Currently, to give structure_consts in first form, must also give basis in first form.

    #OLD:
    Example:
    my_pmc = split_matching(1)
    rho2=Strand_Diagram(my_pmc, [(1,2)],left_idem=[(1,3)])
    rho3=Strand_Diagram(my_pmc, [(2,3)],left_idem=[(0,2)])
    i02=Strand_Diagram(my_pmc, [], left_idem=[(0,2)])
    gens = {'x':[(1,3)],'a':[(0,2)], 'b':[(0,2)]}
    str_consts={'x':[([rho2],'a')], 'a':[], 'b':[([rho3],'x'),([i02],'a')]}
    CFD = TypeDStr(my_pmc, basis=gens, structure_consts=str_consts)
    CFD
    CFD.display()
    CFD.is_bounded()
    
    rho23=Strand_Diagram(my_pmc, [(1,3)], left_idem=[(1,3)])
    new_gens = {'x':[(1,3)]}
    new_str_consts={'x':[([rho23],'x')]}
    new_CFD = TypeDStr(my_pmc,basis=new_gens, structure_consts=new_str_consts)
    """

    def __init__(self, pmc, basis, structure_consts={}):
        self.pmc=pmc
        #Coerce first form of basis to second form.
        if type(basis) == type({0:1,2:3}):
            self.basis = list()
            for x in basis.keys():
                self.basis.append(DGen(x,self.pmc,basis[x]))
        else:
            if type(basis) == type([1,2,3,4]):
                self.basis = basis
            else:
                raise Exception("Basis must be either a dict or a list.")
        #Coerce first form of structure_consts to second.
        self.structure_consts = dict()
        for x in structure_consts.keys():
            if type(structure_consts[x])==type([1,2,3,4]):
                dx = DElt({},self.pmc)
                for (a,y) in structure_consts[x]:
                    dx = dx+DElt({y:a})
                if type(basis)==type({0:1,2:3}):
                    self.structure_consts[DGen(x,self.pmc,basis[x])]=dx
                else:
                    self.structure_consts[x]=dx
            else:
                if type(basis)==type({0:1,2:3}):
                    self.structure_consts[DGen(x,self.pmc,basis[x])]=structure_consts[x]
                else:
                    self.structure_consts[x]=structure_consts[x]
        #Currently does almost no sanity checking; improve.

    def __repr__(self):
        return 'Type D structure with generators '+repr(self.basis)+'\nDelta1 given by '+repr(self.structure_consts)

    def graph(self):
        "Returns a graph representing self."
        verts = self.basis
        edges = dict()
        for v in verts:
            targets = dict()
            for e in self.structure_consts[v].keys():
                targets[repr(e)]=repr(self.structure_consts[v].coeff(e))
#                targets[e]=repr(self.structure_consts[v].coeff(e))
#            edges[v]=targets #Fails with prev two lines for mysterious reasons -- something about __trunc__
            edges[repr(v)]=targets
        G=DiGraph(edges)
        return G


    def show(self):
        self.graph().show(edge_labels=True)

    def is_bounded(self):
        "Returns True if self is bounded and False otherwise."
        return (not self.graph().has_loops())

    def bounding_num(self):
        "Returns the highest n so that deltan is nonzero."
        return longest_path_len(self.graph())

    def delta1(self, elt):
        if elt in self.structure_consts.keys():
            return self.structure_consts[elt]
        if DGen(elt) in self.structure_consts.keys():
            return self.structure_consts[DGen(elt)]

    def olddelta1(self, elt):
        "Returns delta_1(x), as a list (sum) of tuples (a,y)."
        newdel = self.delta1(elt)
        answer = list()
        for x in newdel.keys():
            for a in newdel.coeffs(x):
                answer.append((a,x))
        return answer

    def deltan(self, elt, n):
        "Returns delta_n(x), as a list (sum) of tuples (a_1,a_2,...,a_n,y)."
        if n==0:
            return [([],elt)]
        if n==1:
            return self.olddelta1(elt)
        else:
            almost = self.olddeltan(elt, n-1)
            answer = list()
            for s in almost:
                for t in self.olddelta1(s[n-1]):
                    answer.append(s[:n-1]+t)
        return answer

    def delta(self, elt):
        if not self.is_bounded():
            raise Exception("Type D structure not bounded, so delta not a finite sum.")
        answer = list()
        for i in range(self.bounding_num()+1):
            answer.extend(self.deltan(elt,i))
        return answer

    def simplify(self, keep_bounded=False):
        """Cancel out naked differentias in delta_1. If keep_bounded is True, do this only as much as preserves boundedness.

        Note: keep_bounded option not yet implemented.
        """
        while self.cancel_an_edge(keep_bounded):
            pass

    def cancel_an_edge(self, keep_bounded=False):
        """Find an edge and cancel it. Return True if an edge was found, False if not.

        Note: keep_bounded option not yet implemented.
        """
        for x in self.basis:
            for y in self.structure_consts[x].keys():
                if self.structure_consts[x][y]*self.structure_consts[x][y]==self.structure_consts[x][y]:
                    self.cancel_edge(x,y)
                    return True
        return False

    def cancel_edge(self, source, target):
        """Cancel the edge from source to tagert.

        Should be able to do this if label is a unit, but currently can only cancel if label is an idempotent.
        """
        if self.delta1(source)[target] != self.delta1(source)[target]*self.delta1(source)[target]:
            raise Exception("Can only cancel an edge labeled by an idempotent.")
        #Find the edges into target
        updated_consts = dict(self.structure_consts)
        del updated_consts[source]
        del updated_consts[target]
        otherverts = list(self.basis)
        otherverts.remove(source)
        otherverts.remove(target)
        for x in otherverts:
            if target in self.structure_consts[x].keys():
                updated_consts[x]=updated_consts[x]+self.structure_consts[x][target]*self.structure_consts[source]
        #Finally, delete all references to edges we've deleted
        for x in otherverts:
            if source in updated_consts[x].keys():
                del updated_consts[x][source]
            if target in updated_consts[x].keys():
                del updated_consts[x][target]
        self.basis = otherverts
        self.structure_consts=updated_consts

    def d_sq_zero(self):
        """Returns True is self satisfies d^2=0 and False otherwise.

        Example:
        sage: small_pmc = split_matching(1)
        sage: rho2=Strand_Diagram(small_pmc, [(1,2)],left_idem=[(1,3)])
        sage: rho3=Strand_Diagram(small_pmc, [(2,3)],left_idem=[(0,2)])
        sage: iota02=Strand_Diagram(small_pmc, [], left_idem=[(0,2)])
        sage: gens = {'x':[(1,3)],'a':[(0,2)], 'b':[(0,2)]}
        sage: bad_str_consts={'x':[([rho2],'a')], 'a':[], 'b':[([rho1],'x'),([iota02],'a')]}
        sage: bad_CFD = TypeDStr(small_pmc, basis=gens, structure_consts=bad_str_consts)
        sage: bad_CFD.d_sq_zero()
        False
        sage: my_pmc = split_matching(2)
        sage: newbad_consts={'x':[([Strand_Diagram(my_pmc,[(0,6),(1,3)],[(0,2),(1,3)])],'x')]}
        sage: newbad_CFD = TypeDStr(my_pmc, basis={'x':[(0,2),(1,3)]},structure_consts=newbad_consts)
        """
        #Hasn't been tested much; probably doesn't work quite right.
        for x in self.basis:
            first_part = self.delta1(x).diff_coeffs()
            second_part = DElt({},self.pmc)
            for y in self.delta1(x).keys():
                second_part = second_part + self.delta1(x).coeff(y)*self.delta1(y)
            if first_part + second_part != DElt({},self.pmc):
                return False
        return True

    def shorten_names(self):
        "Replace the names in basis by short ones."
        newbasis = list()
        old_to_new = dict()
        new_to_old = dict()
        for i in range(len(self.basis)):
            ithnewelt = DGen(i,pmc=self.pmc,idem=self.basis[i].idem)
            newbasis.append(ithnewelt)
            old_to_new[self.basis[i]]=ithnewelt
            new_to_old[ithnewelt]=self.basis[i]
        new_str_consts = dict()
        for x in self.structure_consts.keys():
            new_dx_data = dict()
            for y in self.structure_consts[x].keys():
                new_dx_data[old_to_new[y]]=self.structure_consts[x][y]
            new_str_consts[old_to_new[x]]=DElt(new_dx_data,self.pmc)
        self.basis = newbasis
        self.structure_consts = new_str_consts

    def mor_to_d(self, other):
        """Returns the chain complex of homomorphisms from self to other, where other is a type D structure.
        
        Examples:
        sage: infty_type_D(1).mor_to_d(infty_type_D(1))
        Digraph on 2 vertices
        sage: infty_type_D(2).mor_to_d(m_one_type_D(2))
        Digraph on 9 vertices
        sage: infty_type_D(2).mor_to_d(m_one_type_D(2)).homology()
        1
        """
        if self.pmc != other.pmc:
            raise Exception("Can only compute Mor's between type D structures over the same algebra.")
        gens = list()
        alg_gens = self.pmc.alg_basis()
        for x in self.basis:
            #Next loop quite inefficient.
            for a in alg_gens:
                for y in other.basis:
                    if (x.idem==a.left_idem) and (y.idem==a.right_idem):
                        gens.append((x,a,y))
                        #The tuple (x,a,y) stands for the map x -> ay, z->0 if z != x.
        diffs = dict()
        for f in gens:
            df = list()
            #d(f(x))
            #First differentiate y in (x,a,y)
            deltay = other.delta1(f[2])
            ady = AlgElt([f[1]])*deltay
            for (b,z) in ady.basis_expansion():
                df.append((f[0],b,z))
            #Now differentiate a.
            for b in f[1].differential():
                df.append((f[0],b,f[2]))
            #f(dx)
            for xp in self.basis:
                for (b,w) in self.delta1(xp).basis_expansion():
                    if w==f[0]:
                        if b*f[1]:
                            df.append((xp,b*f[1],f[2]))
            diffs[f]=reduce_mod_2(df)
        return ChainCx(diffs)

    def hom_to_d(self, other):
        "Returns a basis for the module of type D structure homomorphisms from self to other."
        pass
        
def infty_type_D(k):
    "Returns the standard type D structure for an infinity-framed handlebody of genus k."
    the_pmc = split_matching(k)
    xidems = list()
    for i in range(k):
        xidems.append((4*i+1,4*i+3))
    diffs = list()
    for i in range(k):
        diffs.append(Strand_Diagram(the_pmc, [(4*i+1,4*i+3)], left_idem=xidems))
    return TypeDStr(the_pmc, {'x':xidems}, {'x':[(diffs,'x')]})

def infty_type_D_bd(k):
    "Returns a bounded type D structure for an infinity-framed handlebody of genus k."
    pass

def zero_type_D(k):
    "Returns the standard type D structure for an zero-framed handlebody of genus k."
    the_pmc = split_matching(k)
    xidems = list()
    for i in range(k):
        xidems.append((4*i,4*i+2))
    diffs = list()
    for i in range(k):
        diffs.append(Strand_Diagram(the_pmc, [(4*i,4*i+2)], left_idem=xidems))
    return TypeDStr(the_pmc, {'y':xidems}, {'y':[(diffs,'y')]})

def zero_type_D_bd(k):
    "Returns a bounded type D structure for an zero-framed handlebody of genus k."
    pass

def m_one_type_D(k):
    "Returns the standard type D structure for a -1-framed handlebody of genus k."
    the_pmc = split_matching(k)
    gens = [[]]
    for i in range(k):
        new_gens=list()
        for x in gens:
            new_gens.append(x+[(4*i,4*i+2)])
            new_gens.append(x+[(4*i+1,4*i+3)])
        gens = new_gens
    diffs = dict()
    for x in gens:
        ans = list()
        for i in range(k):
            if x[i]==(4*i,4*i+2):
                appendme = list(x)
                appendme[i] = (4*i+1,4*i+3)##EDIT HERE.
                ans.append(([Strand_Diagram(the_pmc, [(4*i,4*i+1)], left_idem=x), Strand_Diagram(the_pmc,[(4*i+2,4*i+3)],left_idem=x)],gens.index(appendme)))
        diffs[gens.index(x)]=ans
    basis = dict()
    for x in gens:
        basis[gens.index(x)] = x
    return TypeDStr(split_matching(k),basis,diffs)



#Note that TypeDDStr is a left-left module.


class DDGen():
    """A class for generators of a type DD structure.

    Inputs:
    name: name for this generator. Probably needs to be hashable, and unique.
    pmc_1: underlying first pointed matched circle.
    pmc_2: underlying second pointed matched circle. Defaults to pmc_1 if not given.
    idem_1: first idempotent for this generator.
    idem_2: second idempotent for this generator.

    All data should be given EXCEPT if just want to obtain an element with a given name.

    __eq__ JUST LOOKS AT THE NAME.  Similarly __hash__.

    Example:
    x = DDGen('x',split_matching(2), split_matching(2), [(0,2),(1,3)],[(0,2),(4,6)])
    """
    def __init__(self, name, pmc_1=[], pmc_2=[], idem_1=[], idem_2=[]):
        #Should do some sanity checking.
        self.name = name
        self.pmc_1 = pmc_1
        if not pmc_2:
            self.pmc_2=pmc_1
        else:
            self.pmc_2=pmc_2
        self.idem_1=idem_1
        self.idem_2=idem_2
        self.idem_1.sort()
        self.idem_2.sort()

    def __repr__(self):
        return self.name.__repr__()
    
    def __eq__(self, other):
        "Checks if the NAMES of self and other are the same."
        if not isinstance(other, DDGen):
            return False
        if self.name == other.name:
            return True
        return False

    def __ne__(self, other):
        return not (self.__eq__(other))

    def __hash__(self):
        return self.name.__hash__()

    def exchangeLR(self):
        "Returns result of exchanging pmc_1 and pmc_2, i.e., a\otimes b becomes b\otimes a."
        return DDGen(self.name, self.pmc_2, self.pmc_1, self.idem_2, self.idem_1)

class DDElt(UserDict):
    """A class for elements of a left Type DD structure.

    Input:
    A dict 'data' with keys x_i DDGen's and values AlgBlgElt's (the coefficients of the DDGen's).
    pmc_1: optional argument, giving the first pointed matched circle for this element. Must be given if dict is empty.
    pmc_2: optional argument, giving the second pointed matched circle for this element. Must be given if dict is empty.

    Examples:
    """
    def __init__(self, data, pmc_1=[], pmc_2=[]):
        UserDict.__init__(self, data)
        if pmc_1:
            self.pmc_1 = pmc_1
        else:
            self.pmc_1 = self.data.keys()[0].pmc_1
        if pmc_2:
            self.pmc_2 = pmc_2
        else:
            self.pmc_2 = self.data.keys()[0].pmc_2
        self.reduce()

    def coeff(self, x):
        return self.data[x]

    def diff_1_coeffs(self):
        "Returns the element obtained by differentiating the coefficients on the first side of self."
        answer = dict()
        for x in self.data.keys():
            answer[x] = self.data[x].diff_1()
        return DDElt(answer, pmc_1=self.pmc_1,pmc_2=self.pmc_2)

    def diff_2_coeffs(self):
        "Returns the element obtained by differentiating the coefficients on the second side of self."
        answer = dict()
        for x in self.data.keys():
            answer[x] = self.data[x].diff_2()
        return DDElt(answer, pmc_1=self.pmc_1,pmc_2=self.pmc_2)

    def basis_expansion(self):
        "Returns a list of triples [(rho_i,sigma_i,x_i)], with rho_i and sigma_i are Strand_Diagram and x_i a DDGen so that self is sum_i rho_i\otimes sigma_i \otimes x_i."
        self.reduce()
        answer = list()
        for x in self.data.keys():
            for (a,b) in self.data[x].basis_expansion():
                answer.append((a,b,x))
        return answer

    def __repr__(self):
        return repr(self.data)

    def reduce(self):
        "Delete duplicates from self, and also keys with values the empty list."
        for x in self.data.keys():            
        #Eliminate terms where idempotents don't match up.
            self.data[x] = self.data[x]*AlgBlgElt({Strand_Diagram(self.pmc_1,[],x.idem_1):AlgElt([Strand_Diagram(self.pmc_2,[],x.idem_2)])})
            #Now delete pairs of same term.
            self.data[x].reduce_mod_2()
            if self.data[x]==[]:
                del self.data[x]

    def __add__(self, other):
        if self.pmc_1 != other.pmc_1:
            raise Exception("Can't add DDElts with different pmc_1's.")
        if self.pmc_2 != other.pmc_2:
            raise Exception("Can't add DDElts with different pmc_2's.")
        ansdict = dict()
        for x in self.data.keys():
            if not (x in other.data.keys()):
                ansdict[x]=self.data[x]
            else:
                ansdict[x] = self.data[x]+other.data[x]
        for x in other.data.keys():
            if not (x in self.data.keys()):
                ansdict[x]=other.data[x]
        return DDElt(ansdict, self.pmc_1, self.pmc_2)
    
    def __rmul__(self, other):
        "Left multiply an element by an AlgBlgElt."
        ansdict = dict()
        for x in self.data.keys():
            ansdict[x] = other*self.data[x]
        return DDElt(ansdict, self.pmc_1,self.pmc_2)

    def __eq__(self, other):
        if not isinstance(other, DDElt):
            return False
        if Set(self.data.keys()) != Set(other.data.keys()):
            return False
        for x in self.data.keys():
            if self.data[x] != other.data[x]:
                return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

    def exchangeLR(self):
        "Returns result of exchanging pmc_1 and pmc_2, i.e., a\otimes b becomes b\otimes a."
        answer = dict()
        for x in self.data.keys():
            answer[x.exchangeLR()]=self.data[x].exchangeLR()
        return DDElt(answer, self.pmc_2, self.pmc_1)

#    def __hash__(self):
#        pass

class TypeDDStr():
    """A type DD structure.

    Inputs: 
    --pmc_1: first underlying pointed matched circle.
    --pmc_2: second underlying pointed matched circle.
    --basis: a list of DDGen's
    --structure_consts: a dictionary with keys basis elements and values DDElt's

    Examples:
    #CFDD(Id) for torus boundary:
    p = DDGen('p',small_pmc, small_pmc, [(0,2)], [(0,2)])
    q = DDGen('q',small_pmc, small_pmc, [(1,3)], [(1,3)])
    chords = AlgBlgElt({rho1:AlgElt([rho1]),rho2:AlgElt([rho2]),rho3:AlgElt([rho3]),rho123:AlgElt([rho123])})
    i02chords = AlgBlgElt({iota02:AlgElt([iota02])})*chords
    i13chords = AlgBlgElt({iota13:AlgElt([iota13])})*chords
    TorusId = TypeDDStr(small_pmc, small_pmc, [p,q], {p:DDElt({p:i02chords,q:i02chords}),q:DDElt({p:i13chords,q:i13chords})})
    """

    def __init__(self, pmc_1, pmc_2, basis, structure_consts={}):
        self.pmc_1=pmc_1
        self.pmc_2=pmc_2
        self.basis = basis
        self.structure_consts = structure_consts
        #Should do some sanity checking.

    def __repr__(self):
        parsedstrconsts = str()
        for x in self.structure_consts.keys():
            parsedstrconsts=parsedstrconsts + repr(x)+" -> "+repr(self.structure_consts[x])+"\n"
        return 'Type DD structure with generators '+repr(self.basis)+'\nDelta1 given by\n'+parsedstrconsts
#        return 'Type DD structure with generators '+repr(self.basis)+'\nDelta1 given by '+repr(self.structure_consts)

    def graph(self):
        "Returns a graph representing self."
        verts = self.basis
        edges = dict()
        for v in verts:
            targets = dict()
            for e in self.structure_consts[v].keys():
                targets[repr(e)]=repr(self.structure_consts[v].data[e])
#                targets[e]=repr(self.structure_consts[v].coeff(e))
#            edges[v]=targets #Fails with prev two lines for mysterious reasons -- something about __trunc__
            edges[repr(v)]=targets
        G=DiGraph(edges)
        return G


    def show(self):
        self.graph().show(edge_labels=True)

    def is_bounded(self):
        "Returns True if self is bounded and False otherwise."
        return (not self.graph().has_loops())

    def is_left_bounded(self):
        "Returns True if self is left bounded and False otherwise."
        pass

    def is_right_bounded(self):
        "Returns True if self is right bounded and False otherwise."
        pass

    def bounding_num(self):
        "Returns the highest n so that deltan is nonzero."
        return longest_path_len(self.graph())

    def delta1(self, elt):
        if elt in self.structure_consts.keys():
            return self.structure_consts[elt]
        if DGen(elt) in self.structure_consts.keys():
            return self.structure_consts[DGen(elt)]

    def d_sq_zero(self):
        "Returns True is self satisfies d^2=0 and False otherwise."
        for x in self.basis:
            dx = self.delta1(x)
            dsqx = DDElt({},self.pmc_1,self.pmc_2)
            #differentiate algebra elements on two sides of delta^1(x)
            dsqx = dsqx+dx.diff_1_coeffs()
            dsqx = dsqx+dx.diff_2_coeffs()
            for y in dx.keys():
                dy = self.delta1(y)
                dsqx = dsqx+dx[y]*dy
            dsqx.reduce()
            if dsqx:
                return False
        return True

    def why_d_sq_not_zero(self):
        "Prints an explanation of why d^2 not zero."
        for x in self.basis:
            dx = self.delta1(x)
            dsqx = DDElt({},self.pmc_1,self.pmc_2)
            #differentiate algebra elements on two sides of delta^1(x)
            dsqx = dsqx+dx.diff_1_coeffs()
            dsqx = dsqx+dx.diff_2_coeffs()
            for y in dx.keys():
                dy = self.delta1(y)
                dsqx = dsqx+dx[y]*dy
            dsqx.reduce()
            if dsqx:
                print "Starting with generator: "+repr(x)
                print "d^2 of generator: "+repr(dsqx)
                print "Differential of generator: "+repr(dx)
                print "Differential of first coeffs of df: "+repr(dx.diff_1_coeffs())
                print "Differential of second coeffs of df: "+repr(dx.diff_2_coeffs())
                for y in dx.keys():
                    print "Differential through "+repr(y)+" contributes: "+repr(dx[y]*self.delta1(y))

    def exchangeLR(self):
        "Returns result of exchanging pmc_1 and pmc_2, a DD module over (pmc_2, pmc_1)."
        answer_basis = list()
        for x in self.basis:
            answer_basis.append(x.exchangeLR())
        answer_consts = dict()
        for x in self.basis:
            answer_consts[x.exchangeLR()]=self.structure_consts[x].exchangeLR()
        return TypeDDStr(self.pmc_2, self.pmc_1, answer_basis, answer_consts)

    #Currently assumes we're in middle SpinC structure.
    def mor_to_d(self, other):
        """Returns the chain complex of homomorphisms over A(pmc_1) from self to other, where other is a type D structure.
        
        Examples:
        """
        if self.pmc_1 != other.pmc:
            raise Exception("Can only compute Mor's between structures over the same algebra.")
        #Compile a basis for Mor
        mor_basis = list()
        for m in self.basis:
            for n in other.basis:
                for a in self.pmc_1.alg_basis():
                    if (Set(m.idem_1)==Set(a.left_idem)) and (Set(n.idem)==Set(a.right_idem)):
                        mor_basis.append(DGen((m,a,n),pmc=self.pmc_2,idem=m.idem_2))
        diffs = dict()
#delta^1 on Mor in three parts.
#First: differentiating the algebra element
        for f in mor_basis:
            diffs[f]=DElt({},pmc=self.pmc_2)
            (m,a,n)=f.name
            for b in a.differential():
                diffs[f]=diffs[f]+DElt({DGen((m,b,n),pmc=self.pmc_2,idem=m.idem_2):AlgElt([Strand_Diagram(self.pmc_2,[],m.idem_2)])})
#Second: differentiating the image (element of other)
#        print diffs
        for f in mor_basis:
            (m,a,n)=f.name
            adn = AlgElt([a])*other.delta1(n)
            for (b,p) in adn.basis_expansion():
                diffs[f] = diffs[f]+DElt({DGen((m,b,p),pmc=self.pmc_2,idem=m.idem_2):AlgElt([Strand_Diagram(self.pmc_2,[],m.idem_2)])})
#Third: differentiating the source (element of self). This is the only part which outputs non-idempotent algebra elements.
#        print diffs
        for f in mor_basis:
            (m,a,n)=f.name
            for l in self.basis:
                dl=self.delta1(l)
                if m in dl.keys():
                    for b in dl[m].keys():
                        if b*a:
                            diffs[f] = diffs[f]+DElt({DGen((l,b*a,n),pmc=self.pmc_2,idem=l.idem_2):dl[m][b]})
#        print diffs
        #Now view PMC as over opposite algebra:
        #First, reverse the idempotents.
        rev_basis = list()
        for f in mor_basis:
            (m,a,n)=f.name
            rev_idem = list()
            for (i,j) in m.idem_2:
                rev_idem.append((4*self.pmc_2.genus-1-j,4*self.pmc_2.genus-1-i))
            rev_basis.append(DGen((m,a,n),pmc=self.pmc_2,idem=rev_idem))
        rev_diffs = dict()
        #Now, reverse the algebra element outputs.
        for f in diffs.keys():
            rev_diffs[f]=diffs[f].opposite()
        return TypeDStr(self.pmc_2.opposite(), rev_basis, rev_diffs)


def change_framing(module, bimodules):
    "Twist the framing on module by taking mor_to_d from the bimodules in list bimodules."
    answer = TypeDStr(module.pmc, module.basis, module.structure_consts)
    for x in bimodules:
        answer = x.mor_to_d(answer)
        answer.simplify()
        answer.shorten_names()
    return answer

def dd_identity(pmc):
    "Returns the type DD identity bimodule associated to pmc."
    generators = list()
    oppositepmc = pmc.opposite()
    #Find a list of generators
    idemps = pmc.idempotents()
    for x in idemps:
        generators.append(DDGen(idemps.index(x),pmc, oppositepmc,x,pmc.complementary_idem(x)))
    #Compile a basis for the diagonal subalgebra
    diag_basis = pmc.zero()**oppositepmc.zero()
    for i in range(4*pmc.genus):
        for j in range(i+1,4*pmc.genus):
            for idem in pmc.idempotents():
                if l_idem_compat(pmc, [(i,j)],idem) and l_idem_compat(oppositepmc, [(4*pmc.genus-j-1,4*pmc.genus-i-1)],pmc.complementary_idem(idem)):
                    diag_basis = diag_basis + AlgElt(Strand_Diagram(pmc,[(i,j)],idem))**AlgElt(Strand_Diagram(oppositepmc,[(4*pmc.genus-j-1,4*pmc.genus-i-1)],pmc.complementary_idem(idem)))
    #Now, differentials...
    differentials = dict()
#    print "Diag basis is"+repr(diag_basis)
    for x in generators:
        dx = DDElt({}, pmc, oppositepmc)
        for y in generators:
            dx = dx + DDElt({y:(AlgElt(Strand_Diagram(pmc,[],x.idem_1))**AlgElt(Strand_Diagram(oppositepmc, [], x.idem_2))*diag_basis*AlgElt(Strand_Diagram(pmc,[],y.idem_1))**AlgElt(Strand_Diagram(oppositepmc, [], y.idem_2)))}, pmc, oppositepmc)
        differentials[x]=dx
    return TypeDDStr(pmc, oppositepmc, generators, differentials)

class Underslide():
    """Class for computing DD modules of underslides.

    Input:
    pmc: The pointed matched circle you're arc-sliding.
    i: the moving endpoint of the arc-slide
    j: the endpoint i is sliding over

    Methods:
    dd_mod: generates and returns the type DD module for this underslide. The DD module is also stored in self.ddstr. Calling dd_mod() only generates it the first time; afterwards, the existing dd structure is returned. (So, subsequent calls are quick.)
    show(): display this arc-slide graphically.
    Lots of others used in the construction (and debugging) of dd_mod.

    Other useful data:
    self.pmc_2: the pointed matched circle resulting from the arc-slide.
    self.pmc_2rev: the orientation-reverse of self.pmc_2.

    Caveats:
    Currently restricts to the central SpinC structure, though this would be relatively easy to change.
    """
    def __init__(self, pmc, i, j=i+1):
        if not pmc.is_underslide(i,j):
            raise Exception("Sliding i over j is not an underslide.")
        self.i = i
        self.j = j
        self.u1chords = None
        self.u2chords = None
        self.u3chords = None
        self.u4chords = None
        self.u5chords = None
        self.u6chords = None
        self.allchords = None
        self.gen_idems = None
        self.generators = None
        self.ddstr = None
    #The PMC resulting from slide:
        self.pmc_1 = pmc
        self.pmc_2 = pmc.arcslide(i,j)
        self.pmc_2rev = self.pmc_2.opposite()
        self.genus = self.pmc_1.genus
    #Notation from HFa:
        self.c1 = j
        self.c2 = pmc.matched_point(self.c1)
        self.b1 = i
        self.b2 = pmc.matched_point(self.b1)
        self.c1p = self.c1
        self.c2p = self.pmc_2.matched_point(self.c1p)
        if self.b1 > self.c1:
            self.b1p = self.c2p-1
        if self.b1 < self.c1:
            self.b1p = self.c2p+1
        self.b2p = self.pmc_2.matched_point(self.b1p)
        
    def is_up_underslide(self):
        if self.j==self.i+1:
            return True
        return False

    def is_down_underslide(self):
        if self.j==self.i-1:
            return True
        return False

    def r(self, x):
        "Take a point a in self.pmc_1 to the corresponding point in self.pmc_2, assuming a is not b_1."
        if x==self.b1:
            raise Exception("r not defined on b1.")
        if (x < self.b1) and (x<self.c2):
            return x
        if (x > self.b1) and (x>self.c2):
            return x
        if self.b1<x<self.c2:
            return x-1
        if self.c2<x<self.b1:
            return x+1
        if x==self.c2:
            return self.c2p

    def rinv(self, x):
        "The inverse of the map r."
        if x==self.b1p:
            raise Exception("r inverse not defined on b1'.")
        if (x < self.b1p) and (x<self.c1p):
            return x
        if (x > self.b1p) and (x>self.c1p):
            return x
        if self.b1p<x<self.c1p:
            return x-1
        if self.c1p<x<self.b1p:
            return x+1
        if x==self.c1p:
            return self.c1

    def generate_U1(self):
        "Generates a list of chords of type U1. Stored in self.u1chords, in the form (xi, xi'), where xi is a list of chords (containing exactly one chord), and similarly xi'."
        answer = list()
        for x in range(4*self.pmc_1.genus):
            for y in range(x+1, 4*self.pmc_1.genus):
                if (x != self.b1) and (y!= self.b1) and (not (x,y) in self.pmc_1.matching) and (not (y,x) in self.pmc_1.matching):
                    answer.append(([(x,y)],[(self.r(x),self.r(y))]))
        self.u1chords = answer

    def generate_U2(self):
        "Generates a list of chords of type U2. Stored in self.u2chords"
        answer = list()
        if self.b1 < self.c1:
            answer.append(([(self.b1,self.c1)],[]))
        if self.b1 > self.c1:
            answer.append(([(self.c1,self.b1)],[]))
        if self.b1p < self.c2p:
            answer.append(([],[(self.b1p,self.c2p)]))
        if self.b1p > self.c2p:
            answer.append(([],[(self.c2p,self.b1p)]))
        self.u2chords = answer

    def generate_U3(self):
        "Generates a list of chords of type U3. Stored in self.u3chords"
        answer = list()
        #Extra sigma
        if self.b1 < self.c1:
            for x in range(self.c1+1,4*self.genus):
                answer.append(([(self.b1,x)],[(self.c1p,self.r(x))]))
        if self.b1 > self.c1:
            for x in range(self.c1):
                answer.append(([(x,self.b1)],[(self.r(x),self.c1p)]))
        #Extra sigma'
        if self.b1p < self.c2p:
            for x in range(self.c2p+1,4*self.genus):
                answer.append(([(self.c2,self.rinv(x))],[(self.b1p,x)]))
        if self.b1p > self.c2p:
            for x in range(self.c2p):
                answer.append(([(self.rinv(x),self.c2)],[(x,self.b1p)]))
        self.u3chords = answer

    def generate_U4(self):
        "Generates a list of chords of type U4. Stored in self.u4chords, in the form (xi, xi'), where xi is a list of chords, and similarly xi'."
        answer = list()
        #Two connected chords
        #Missing sigma:
        if self.b1 < self.c1:
            for x in range(self.b1):
                answer.append(([(x,self.b1)],[(self.r(x),self.c1p)]))
        if self.b1 > self.c1:
            for x in range(self.b1+1, 4*self.genus):
                answer.append(([(self.b1,x)],[(self.c1p,self.r(x))]))
        #Missing sigma':
        if self.b1p < self.c2p:
            for x in range(self.b1p):
                answer.append(([(self.rinv(x),self.c2)],[(x,self.b1p)]))
        if self.b1p > self.c2p:
            for x in range(self.b1p+1, 4*self.genus):
                answer.append(([(self.c2, self.rinv(x))],[(self.b1p,x)]))
        #Three connected chords total, two on left.
        if self.b1 < self.c1:
            for x in range(self.b1):
                for y in range(self.c1+1,4*self.genus):
                    answer.append(([(x,self.b1),(self.c1,y)],[(self.r(x),self.r(y))]))
        if self.b1 > self.c1:
            for x in range(self.c1):
                for y in range(self.b1+1, 4*self.genus):
                    answer.append(([(x,self.c1),(self.b1,y)],[(self.r(x),self.r(y))]))
        #Three connected chords total, two on right.
        if self.b1p < self.c2p:
            for x in range(self.b1p):
                for y in range(self.c2p+1, 4*self.genus):
                    answer.append(([(self.rinv(x),self.rinv(y))],[(x,self.b1p),(self.c2p,y)]))
        if self.b1p > self.c2p:
            for x in range(self.c2p):
                for y in range(self.b1p+1, 4*self.genus):
                    answer.append(([(self.rinv(x),self.rinv(y))],[(x,self.c2p),(self.b1p,y)]))
        self.u4chords = answer

    def generate_U5(self):
        "Generates a list of chords of type U5. Stored in self.u5chords"
        answer = list()
        smaller = min(self.c1, self.c2)
        bigger = max(self.c1,self.c2)
        rbigger = self.r(bigger)
        rsmaller = self.r(smaller)
        for x in range(smaller):
            for y in range(bigger+1, 4*self.genus): #Used to be range(bigger,4*self.genus). Bohua caught the bug.
                answer.append(([(x,smaller),(bigger,y)],[(self.r(x),rsmaller),(rbigger,self.r(y))]))
        self.u5chords = answer

    def generate_U6(self):
        "Generates a list of chords of type U6. Stored in self.u6chords"
        answer = list()
        #Extra sigma:
        if self.b1 < self.c1:
            for x in range(self.c2+1, 4*self.genus):
                if (x != self.b1) and (x != self.c1):
                    answer.append(([(self.c2,x), (self.b1,self.c1)],[(self.b1p, self.r(x))]))
        if self.b1 > self.c1:
            for x in range(self.c2):
                if (x != self.b1) and (x != self.c1):
                    answer.append(([(x, self.c2), (self.c1,self.b1)],[(self.r(x),self.b1p)]))
        #Extra sigma':
        if self.b1p < self.c2p:
            for x in range(self.c1p+1, 4*self.genus):
                if (x != self.b1p) and (x != self.c2p):
                    answer.append(([(self.b1, self.rinv(x))],[(self.c1p,x),(self.b1p,self.c2p)]))
        if self.b1p > self.c2p:
            for x in range(self.c1p):
                if (x != self.b1p) and (x != self.c2p):
                    answer.append(([(self.rinv(x),self.b1)],[(x,self.c1p),(self.c2p,self.b1p)]))
        self.u6chords = answer

    def generate_chords(self):
        "Generates a list of near-chords for self."
        regenallchords = False
        if self.allchords == None:
            self.regenallchords = True
        if self.u1chords == None:
            self.generate_U1()
            regenallchords = True
        if self.u2chords == None:
            self.generate_U2()
            regenallchords = True
        if self.u3chords == None:
            self.generate_U3()
            regenallchords = True
        if self.u4chords == None:
            self.generate_U4()
            regenallchords = True
        if self.u5chords == None:
            self.generate_U5()
            regenallchords = True
        if self.u6chords == None:
            self.generate_U6()
            regenallchords = True
        if regenallchords:
            self.allchords = self.u1chords + self.u2chords + self.u3chords + self.u4chords + self.u5chords + self.u6chords

    def complementary_idem(self, idem):
        "Given an idempotent of self.pmc_1, return the complementary idempotent of self.pmc_2."
        answer = list(self.pmc_2.matching)
        for (x,y) in self.pmc_2.matching:
            if (x != self.b1p) and (y != self.b1p ):
                if ( (self.rinv(x), self.rinv(y)) in idem ) or ( (self.rinv(y), self.rinv(x))  in idem):
                    answer.remove((x,y))
        if ( (self.b1, self.b2) in idem ) or ( (self.b2,self.b1) in idem ):
            if self.b1p < self.b2p:
                answer.remove((self.b1p, self.b2p))
            else:
                answer.remove((self.b2p, self.b1p))
        answer.sort()
        return answer

    def generate_generators(self):
        """Generates a list of generators of the DD module for this arcslide. Stored in self.generators
        Currently restricts to the middle SpinC structure.
        """
        answer = list()
        #Generate gen_idems, if necessary
        if self.gen_idems == None:
            self.generate_gen_idems()
        for (x,y) in self.gen_idems:
            yrev = list()
            for (m,n) in y:
                yrev.append((4*self.genus-n-1, 4*self.genus-m-1))
            answer.append(DDGen(repr(x)+'|'+repr(yrev),self.pmc_1, self.pmc_2rev,x,yrev))
        self.generators = answer

    def generate_gen_idems(self):
        """Generates a list of pairs of idempotents for generators for the DD module. Stored in self.gen_idems
        Currently restricts to the middle SpinC structure.
        """
        #Generators of type X (complementary)
        xidems = list()
        pmc1idems = self.pmc_1.idempotents()
        for idem in pmc1idems:
            xidems.append((idem, self.complementary_idem(idem)))
        #Generators of type Y (sub-complementary)
        yidems = list()
        for idem in pmc1idems:
            if ( (self.c1, self.c2) in idem ) or ( (self.c2, self.c1) in idem):
                if ( not ( (self.b1, self.b2) in idem)) and ( not ( (self.b2, self.b1) in idem) ):
                    scompidem = self.complementary_idem(idem)
                    if self.b1p<self.b2p:
                        scompidem.remove((self.b1p,self.b2p))
                    if self.b2p<self.b1p:
                        scompidem.remove((self.b2p,self.b1p))
                    if self.c1p < self.c2p:
                        scompidem.append((self.c1p,self.c2p))
                    if self.c2p < self.c1p:
                        scompidem.append((self.c2p,self.c1p))
                    scompidem.sort()
                    yidems.append((idem, scompidem))                        
        answer = xidems + yidems
        self.gen_idems = answer

    def dd_mod(self):
        "Return the type DD module for this arcslide."
        if self.ddstr != None:
            return self.ddstr
        if self.generators == None:
            self.generate_generators()
        self.generate_chords()
        diffs = dict()
        for x in self.generators:
            dx = DDElt({}, self.pmc_1, self.pmc_2rev)
            for y in self.generators:
                for (a,b) in self.allchords:
                    #If a is compatible with x's left idempotents and y's left idempotents, and b is compatible with x's right idempotents and y's right idempotents, add corresponding elt to dx.
                    if l_idem_compat(self.pmc_1, a, x.idem_1) and l_idem_compat(self.pmc_2rev, self.opposite_strands(b), x.idem_2):
                        aalgelt = AlgElt(Strand_Diagram(self.pmc_1, a, x.idem_1))*AlgElt(Strand_Diagram(self.pmc_1, [], y.idem_1))
                        balgelt = AlgElt(Strand_Diagram(self.pmc_2rev, self.opposite_strands(b), x.idem_2))*AlgElt(Strand_Diagram(self.pmc_2rev, [], y.idem_2))
                        abalgelt = aalgelt**balgelt
                        if abalgelt:
                            dx = dx + DDElt({y:abalgelt}, self.pmc_1, self.pmc_2rev)
            diffs[x] = dx
        self.ddstr = TypeDDStr(self.pmc_1, self.pmc_2rev, self.generators, diffs)
        return self.ddstr

    def opposite_strands(self, strands):
        answer = list()
        for (x,y) in strands:
            answer.append((4*self.genus-y-1, 4*self.genus-x-1))
        return answer

    def show(self):
        "Display graphically the pair (lchord, rchord). Intended for testing generate_U1() ... generate_U6()."
        #First show the arcslide
        pict = line([(0,-1), (0,4*self.genus)], rgbcolor = (0,0,0))
        pict = pict + line([(2,-1), (2,4*self.genus)], rgbcolor = (0,0,0))
        for i in range(4*self.genus):
            if i != self.b1:
                pict = pict + line([(0,i),(2,self.r(i))], rgbcolor = (1,0,0))
        if self.b1 < self.c1:
            pict = pict + line([(0,self.b1), (1,self.b1+1)], rgbcolor = (1,.25,.25)) + line([(1,self.b1p-1), (2,self.b1p)], rgbcolor = (1,.25,.25))
        if self.b1 > self.c1:
            pict = pict + line([(0,self.b1), (1,self.b1-1)], rgbcolor = (1,.25,.25)) + line([(1,self.b1p+1), (2,self.b1p)], rgbcolor = (1,.25,.25))
        #Show the matchings on the two sides:
        #left:
        for l in range(len(self.pmc_1.matching)):
            pict = pict + line([(-0.2, self.pmc_1.matching[l][0]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][0])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(-0.2, self.pmc_1.matching[l][1]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][1])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(-0.3-l/(8*self.genus), self.pmc_1.matching[l][0]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][1])], rgbcolor = (.2,.2,.2))
        #right:
        for l in range(len(self.pmc_2.matching)):
            pict = pict + line([(2.2, self.pmc_2.matching[l][0]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][0])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(2.2, self.pmc_2.matching[l][1]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][1])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(2.3+l/(8*self.genus), self.pmc_2.matching[l][0]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][1])], rgbcolor = (.2,.2,.2))
        pict.show(axes=False) #, xmin=-2,xmax=2, ymin=-.5, ymax=4*self.genus-.5)


    def display_chord(self, chords):
        "Display graphically the pair (lchord, rchord). Intended for testing generate_U1() ... generate_U6()."
        lchord = chords[0]
        rchord = chords[1]
        #First show the arcslide
        pict = line([(0,-1), (0,4*self.genus)], rgbcolor = (0,0,0))
        pict = pict + line([(2,-1), (2,4*self.genus)], rgbcolor = (0,0,0))
        for i in range(4*self.genus):
            if i != self.b1:
                pict = pict + line([(0,i),(2,self.r(i))], rgbcolor = (1,0,0))
        if self.b1 < self.c1:
            pict = pict + line([(0,self.b1), (1,self.b1+1)], rgbcolor = (1,.25,.25)) + line([(1,self.b1p-1), (2,self.b1p)], rgbcolor = (1,.25,.25))
        if self.b1 > self.c1:
            pict = pict + line([(0,self.b1), (1,self.b1-1)], rgbcolor = (1,.25,.25)) + line([(1,self.b1p+1), (2,self.b1p)], rgbcolor = (1,.25,.25))
        #light lines going farther out, for clarity.
        for x in range(4*self.genus):
            pict = pict + line([(-1,x),(0,x)], rgbcolor = (.8,.8,.8))
            pict = pict + line([(2,x),(3,x)], rgbcolor = (.8,.8,.8))
        #Show the matchings on the two sides:
        #left:
        for l in range(len(self.pmc_1.matching)):
            pict = pict + line([(-0.2, self.pmc_1.matching[l][0]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][0])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(-0.2, self.pmc_1.matching[l][1]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][1])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(-0.3-l/(8*self.genus), self.pmc_1.matching[l][0]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][1])], rgbcolor = (.2,.2,.2))
        #right:
        for l in range(len(self.pmc_2.matching)):
            pict = pict + line([(2.2, self.pmc_2.matching[l][0]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][0])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(2.2, self.pmc_2.matching[l][1]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][1])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(2.3+l/(8*self.genus), self.pmc_2.matching[l][0]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][1])], rgbcolor = (.2,.2,.2))
        #Now draw the chord on the two sides.
        for (x,y) in lchord:
            pict = pict + line([(-1,x),(-0.1, y)], rgbcolor = (0,0,1))
        for (x,y) in rchord:
            pict = pict + line([(2.1,x),(3, y)], rgbcolor = (0,0,1))
        pict.show(axes=False) #, xmin=-2,xmax=2, ymin=-.5, ymax=4*self.genus-.5)

class Overslide():
    """Class for computing DD modules of overslides.

    Input:
    pmc: The pointed matched circle you're arc-sliding.
    i: the moving endpoint of the arc-slide
    j: the endpoint i is sliding over

    Methods:
    dd_mod: generates and returns the type DD module for this underslide. The DD module is also stored in self.ddstr. Calling dd_mod() only generates it the first time; afterwards, the existing dd structure is returned. (So, subsequent calls are quick.)
    show(): display this arc-slide graphically.
    Lots of others used in the construction (and debugging) of dd_mod.

    Other useful data:
    self.pmc_2: the pointed matched circle resulting from the arc-slide.
    self.pmc_2rev: the orientation-reverse of self.pmc_2.

    Caveats:
    Currently restricts to the central SpinC structure, though this would be relatively easy to change.
    """
    def __init__(self, pmc, i, j=i+1):
        if not pmc.is_overslide(i,j):
            raise Exception("Sliding i over j is not an overslide.")
        self.i = i
        self.j = j
        self.o1DetChords = None
        self.o2DetChords = None
        self.o3DetChords = None
        self.o4DetChords = None
        self.o5DetChords = None
        self.o6DetChords = None
        self.o3IndetChords = None
        self.o4IndetChords = None
        self.o7IndetChords = None
        self.o8IndetChords = None
        self.DetChords = None
        self.IndetChords = None
        self.gen_idems = None
        self.generators = None
        self.ddstr = None
    #The PMC resulting from slide:
        self.pmc_1 = pmc
        self.pmc_2 = pmc.arcslide(i,j)
        self.pmc_2rev = self.pmc_2.opposite()
        self.genus = self.pmc_1.genus
    #Notation from HFa:
        self.c1 = j
        self.c2 = pmc.matched_point(self.c1)
        self.b1 = i
        self.b2 = pmc.matched_point(self.b1)
        if self.b1>self.c1:
            self.c1p = self.c1+1
        if self.b1<self.c1:
            self.c1p=self.c1-1
        self.c2p = self.pmc_2.matched_point(self.c1p)
        if self.b1 > self.c1:
            self.b1p = self.c2p-1
        if self.b1 < self.c1:
            self.b1p = self.c2p+1
        self.b2p = self.pmc_2.matched_point(self.b1p)
        
    def is_up_overslide(self):
        if self.j==self.i+1:
            return True
        return False

    def is_down_overslide(self):
        if self.j==self.i-1:
            return True
        return False

    def r(self, x):
        "Take a point a in self.pmc_1 to the corresponding point in self.pmc_2, assuming a is not b_1."
        if x==self.b1:
            raise Exception("r not defined on b1.")
        if (x < self.b1) and (x<self.c2):
            return x
        if (x > self.b1) and (x>self.c2):
            return x
        if self.b1<x<self.c2:
            return x-1
        if self.c2<x<self.b1:
            return x+1
        if x==self.c2:
            return self.c2p

    def rinv(self, x):
        "The inverse of the map r."
        if x==self.b1p:
            raise Exception("r inverse not defined on b1'.")
        if (x < self.b1p) and (x<self.c1p):
            return x
        if (x > self.b1p) and (x>self.c1p):
            return x
        if self.b1p<x<self.c1p:
            return x-1
        if self.c1p<x<self.b1p:
            return x+1
        if x==self.c1p:
            return self.c1

    def generate_O1(self):
        "Generates a list of chords of type O1. Stored in self.o1DetChords (since they're all determinate), in the form (xi, xi'), where xi is a list of chords (containing exactly one chord), and similarly xi'."
        answer = list()
        for x in range(4*self.pmc_1.genus):
            for y in range(x+1, 4*self.pmc_1.genus):
                if (x != self.b1) and (y!= self.b1) and (not (x,y) in self.pmc_1.matching) and (not (y,x) in self.pmc_1.matching):
                    answer.append(([(x,y)],[(self.r(x),self.r(y))]))
        self.o1DetChords = answer

    def generate_O2(self):
        "Generates a list of chords of type O2. Stored in self.O2DetChords"
        answer = list()
        if self.b1 < self.c1:
            answer.append(([(self.b1,self.c1)],[]))
        if self.b1 > self.c1:
            answer.append(([(self.c1,self.b1)],[]))
        if self.b1p < self.c2p:
            answer.append(([],[(self.b1p,self.c2p)]))
        if self.b1p > self.c2p:
            answer.append(([],[(self.c2p,self.b1p)]))
        self.o2DetChords = answer

    def generate_O3(self):
        "Generates a list of chords of type O3. Stored in self.o3DetChords and self.o3IndetChords, depending on whether the chord is determinate or not."
        #The determinate ones:
        answer = list()
        #Extra sigma
        if self.b1 < self.c1:
            for x in range(self.c1+1,4*self.genus):
                if x != self.c2:
                    answer.append(([(self.b1,x)],[(self.c1p,self.r(x))]))
        if self.b1 > self.c1:
            for x in range(self.c1):
                if x!= self.c2:
                    answer.append(([(x,self.b1)],[(self.r(x),self.c1p)]))
        #Extra sigma'
        if self.b1p < self.c2p:
            for x in range(self.c2p+1,4*self.genus):
                if x!= self.c1p:
                    answer.append(([(self.c2,self.rinv(x))],[(self.b1p,x)]))
        if self.b1p > self.c2p:
            for x in range(self.c2p):
                if x!= self.c1p:
                    answer.append(([(self.rinv(x),self.c2)],[(x,self.b1p)]))
        self.o3DetChords = answer
        #Now, the indeterminate ones
        indet = list()
        if self.b1<self.c1:
            indet.append(([(self.b1, self.c2)],[(self.c1p, self.c2p)]))
            indet.append(([(self.c1, self.c2)],[(self.c1p, self.b1p)]))
        if self.b1>self.c1:
            indet.append(([(self.c2, self.b1)],[(self.c2p, self.c1p)]))
            indet.append(([(self.c2, self.c1)],[(self.b1p, self.c1p)]))
        self.o3IndetChords = indet

    def generate_O4(self):
        "Generates a list of chords of type U4. Stored in self.o4DetChords and self.o4IndetChords, in the form (xi, xi'), where xi is a list of chords, and similarly xi'."
        detanswer = list()
        indetanswer = list()
        #Two connected chords
        #Missing sigma:
        if self.b1 < self.c1:
            for x in range(self.b1):
                detanswer.append(([(x,self.b1)],[(self.r(x),self.c1p)]))
        if self.b1 > self.c1:
            for x in range(self.b1+1, 4*self.genus):
                detanswer.append(([(self.b1,x)],[(self.c1p,self.r(x))]))
        #Missing sigma':
        if self.b1p < self.c2p:
            for x in range(self.b1p):
                detanswer.append(([(self.rinv(x),self.c2)],[(x,self.b1p)]))
        if self.b1p > self.c2p:
            for x in range(self.b1p+1, 4*self.genus):
                detanswer.append(([(self.c2, self.rinv(x))],[(self.b1p,x)]))
        #Three connected chords total, two on left.
        if self.b1 < self.c1:
            for x in range(self.b1):
                for y in range(self.c1+1,4*self.genus):
                    #Is this indeterminate?
                    if y==self.c2:
                        indetanswer.append(([(x,self.b1),(self.c1,y)],[(self.r(x),self.r(y))]))
                    else:
                        detanswer.append(([(x,self.b1),(self.c1,y)],[(self.r(x),self.r(y))]))                        
        if self.b1 > self.c1:
            for x in range(self.c1):
                for y in range(self.b1+1, 4*self.genus):
                    #Is this indeterminate?
                    if x==self.c2:
                        indetanswer.append(([(x,self.c1),(self.b1,y)],[(self.r(x),self.r(y))]))                        
                    else:
                        detanswer.append(([(x,self.c1),(self.b1,y)],[(self.r(x),self.r(y))]))
        #Three connected chords total, two on right.
        if self.b1p < self.c2p:
            for x in range(self.b1p):
                for y in range(self.c2p+1, 4*self.genus):
                    #Is this indeterminate?
                    if y==self.c1p:
                        indetanswer.append(([(self.rinv(x),self.rinv(y))],[(x,self.b1p),(self.c2p,y)]))
                    else:
                        detanswer.append(([(self.rinv(x),self.rinv(y))],[(x,self.b1p),(self.c2p,y)]))
        if self.b1p > self.c2p:
            for x in range(self.c2p):
                for y in range(self.b1p+1, 4*self.genus):
                    #Is this indeterminate?
                    if x==self.c1p:
                        indetanswer.append(([(self.rinv(x),self.rinv(y))],[(x,self.c2p),(self.b1p,y)]))
                    else:
                        detanswer.append(([(self.rinv(x),self.rinv(y))],[(x,self.c2p),(self.b1p,y)]))
        self.o4DetChords = detanswer
        self.o4IndetChords = indetanswer

    def generate_O5(self):
        "Generates a list of chords of type O5. Stored in self.o5DetChords"
        answer = list()
        smaller = min(self.c1, self.c2)
        bigger = max(self.c1,self.c2)
        rbigger = self.r(bigger)
        rsmaller = self.r(smaller)
        #Chords can be disjoint:
        for x in range(smaller+1,bigger):
            for y in range(x+1,bigger):
                answer.append(([(smaller,x),(y,bigger)],[(rsmaller,self.r(x)),(self.r(y),rbigger)]))
        #Or one can be contained in the other:
        for x in range(bigger,4*self.genus):
            for y in range(smaller+1,bigger):
                if (x != self.b1) and (x != self.b2):
                    answer.append(([(smaller,x),(y,bigger)],[(rsmaller,self.r(x)),(self.r(y),rbigger)]))
        for x in range(smaller+1,bigger):
            for y in range(smaller):
                if (y != self.b1) and (y != self.b2):
                    answer.append(([(smaller,x),(y,bigger)],[(rsmaller,self.r(x)),(self.r(y),rbigger)]))
        self.o5DetChords = answer

    def generate_O6(self):
        "Generates a list of chords of type O6. Stored in self.o6DetChords"
        answer = list()
        #Extra sigma:
        if self.b1 > self.c1:
            for x in range(self.c2):
                answer.append(([(x, self.c2),(self.c1,self.b1)],[(self.r(x),self.b1p)]))
        if self.b1 < self.c1:
            for x in range(self.c2+1,4*self.genus):
                answer.append(([(self.b1,self.c1),(self.c2,x)],[(self.b1p,self.r(x))]))

        #Extra sigma'
        if self.b1 > self.c1:
            for x in range(self.b1+1,4*self.genus):
                answer.append(([(self.b1, x)],[(self.c1p,self.r(x)),(self.b1p,self.c2p)]))
        if self.b1 < self.c1:
            for x in range(self.b1):
                answer.append(([(x,self.b1)],[(self.r(x),self.c1p),(self.c2p,self.b1p)]))
        self.o6DetChords = answer

    def generate_O7(self):
        "Generates a list of chords of type O7. Stored in self.o7IndetChords"   
        answer = list()
        #Break on left.
        if self.c1<self.c2:
            for x in range(self.c1+1,self.c2):
                answer.append(([(self.c1,x),(x,self.c2)],[(self.c1p,self.c2p)]))
        if self.c2<self.c1:
            for x in range(self.c2+1,self.c1):
                answer.append(([(self.c2,x),(x,self.c1)],[(self.c2p,self.c1p)]))
        #Break on right.
        if self.c1p<self.c2p:
            for x in range(self.c1p+1,self.c2p):
                answer.append(([(self.c1,self.c2)],[(self.c1p,x),(x,self.c2p)]))
        if self.c2p<self.c1p:
            for x in range(self.c2p+1,self.c1p):
                answer.append(([(self.c2,self.c1)],[(self.c2p,x),(x,self.c1p)]))
        self.o7IndetChords = answer

    def generate_O8(self):
        "Generates a list of chords of type O7. Stored in self.o7IndetChords"   
        answer = list()
        #xi contained in [c1,c2]
        smaller = min(self.c1, self.c2)
        bigger = max(self.c1, self.c2)
        rsmaller = min(self.c1p, self.c2p)
        rbigger = max(self.c1p, self.c2p)
        for x in range(smaller+1,bigger):
            for y in range(x+1,bigger):
                if (not (x,y) in self.pmc_1.matching) and (not (y,x) in self.pmc_1.matching):
                    answer.append(([(x,y),(smaller,bigger)],[(self.r(x),self.r(y)),(rsmaller,rbigger)]))
        #xi disjoint from [c1,c2]
        #xi below than c1:
        for x in range(smaller):
            for y in range(x+1,smaller):
                if (not (x,y) in self.pmc_1.matching) and (not (y,x) in self.pmc_1.matching) and (x!=self.b1) and (y!=self.b1):
                    answer.append(([(x,y),(smaller,bigger)],[(self.r(x),self.r(y)),(rsmaller,rbigger)]))
        #xi above c2:
        for x in range(bigger+1, 4*self.genus):
            for y in range(x+1, 4*self.genus):
                if (not (x,y) in self.pmc_1.matching) and (not (y,x) in self.pmc_1.matching) and (x != self.b1) and (y !=self.b1):
                    answer.append(([(x,y),(smaller,bigger)],[(self.r(x),self.r(y)),(rsmaller,rbigger)]))
        #NOTE: for idempotent reasons, chords of this type with an endpoint on b1 or b2 wont count. But they have not been explicitly disallowed above.
        self.o8IndetChords = answer

    def generate_chords(self):
        "Generates a list of near-chords for self."
        regenallchords = False
        if (self.DetChords == None) or (self.Indetchords == None) :
            regenallchords =  True
        if self.o1DetChords == None:
            self.generate_O1()
            regenallchords = True
        if self.o2DetChords == None:
            self.generate_O2()
            regenallchords = True
        if self.o3DetChords == None:
            self.generate_O3()
            regenallchords = True
        if self.o4DetChords == None:
            self.generate_O4()
            regenallchords = True
        if self.o5DetChords == None:
            self.generate_O5()
            regenallchords = True
        if self.o6DetChords == None:
            self.generate_O6()
            regenallchords = True
        if self.o7IndetChords == None:
            self.generate_O7()
            regenallchords = True
        if self.o8IndetChords == None:
            self.generate_O8()
            regenallchords = True
        if regenallchords:
            self.DetChords = self.o1DetChords+self.o2DetChords+self.o3DetChords+self.o4DetChords+self.o5DetChords+self.o6DetChords
            self.IndetChords = self.o3IndetChords+self.o4IndetChords+self.o7IndetChords+self.o8IndetChords

    def basic_choice(self):
        "Generate the indeterminate chords corresponding to a particular basic chioce."
        answer = list()
        #The indeterminate O3 chords: those using sigma', not those using sigma.
        if self.b1<self.c1:
            answer.append(([(self.c1, self.c2)],[(self.c1p, self.b1p)]))
        if self.b1>self.c1:
            answer.append(([(self.c2, self.c1)],[(self.b1p, self.c1p)]))
        #The indeterminate O4 chords
        #Three connected chords total, two on left.
        if self.b1 < self.c1:
            for x in range(self.b1):
                answer.append(([(x,self.b1),(self.c1,self.c2)],[(self.r(x),self.c2p)]))
        if self.b1 > self.c1:
            for y in range(self.b1+1, 4*self.genus):
                answer.append(([(self.c2,self.c1),(self.b1,y)],[(self.c2p,self.r(y))]))                        
        #The O7 chords: those with a break on the left.
        if self.c1<self.c2:
            for x in range(self.c1+1,self.c2):
                answer.append(([(self.c1,x),(x,self.c2)],[(self.c1p,self.c2p)]))
        if self.c2<self.c1:
            for x in range(self.c2+1,self.c1):
                answer.append(([(self.c2,x),(x,self.c1)],[(self.c2p,self.c1p)]))
        #No O8 chords.
        self.choice_chords = answer

    def complementary_idem(self, idem):
        "Given an idempotent of self.pmc_1, return the complementary idempotent of self.pmc_2."
        answer = list(self.pmc_2.matching)
        for (x,y) in self.pmc_2.matching:
            if (x != self.b1p) and (y != self.b1p ):
                if ( (self.rinv(x), self.rinv(y)) in idem ) or ( (self.rinv(y), self.rinv(x))  in idem):
                    answer.remove((x,y))
        if ( (self.b1, self.b2) in idem ) or ( (self.b2,self.b1) in idem ):
            if self.b1p < self.b2p:
                answer.remove((self.b1p, self.b2p))
            else:
                answer.remove((self.b2p, self.b1p))
        answer.sort()
        return answer

    def generate_generators(self):
        """Generates a list of generators of the DD module for this arcslide. Stored in self.generators
        Currently restricts to the middle SpinC structure.
        """
        answer = list()
        #Generate gen_idems, if necessary
        if self.gen_idems == None:
            self.generate_gen_idems()
        for (x,y) in self.gen_idems:
            yrev = list()
            for (m,n) in y:
                yrev.append((4*self.genus-n-1, 4*self.genus-m-1))
            answer.append(DDGen(repr(x)+'|'+repr(yrev),self.pmc_1, self.pmc_2rev,x,yrev))
        self.generators = answer

    def generate_gen_idems(self):
        """Generates a list of pairs of idempotents for generators for the DD module. Stored in self.gen_idems
        Currently restricts to the middle SpinC structure.
        """
        #Generators of type X (complementary)
        xidems = list()
        pmc1idems = self.pmc_1.idempotents()
        for idem in pmc1idems:
            xidems.append((idem, self.complementary_idem(idem)))
        #Generators of type Y (sub-complementary)
        yidems = list()
        for idem in pmc1idems:
            if ( (self.c1, self.c2) in idem ) or ( (self.c2, self.c1) in idem):
                if ( not ( (self.b1, self.b2) in idem)) and ( not ( (self.b2, self.b1) in idem) ):
                    scompidem = self.complementary_idem(idem)
                    if self.b1p<self.b2p:
                        scompidem.remove((self.b1p,self.b2p))
                    if self.b2p<self.b1p:
                        scompidem.remove((self.b2p,self.b1p))
                    if self.c1p < self.c2p:
                        scompidem.append((self.c1p,self.c2p))
                    if self.c2p < self.c1p:
                        scompidem.append((self.c2p,self.c1p))
                    scompidem.sort()
                    yidems.append((idem, scompidem))                        
        answer = xidems + yidems
        self.gen_idems = answer

    def dd_mod(self):
        "Return the type DD module for this arcslide."
        if self.ddstr != None:
            return self.ddstr
        if self.generators == None:
            self.generate_generators()
        self.generate_chords()
        self.basic_choice()
        diffs = dict()
        for x in self.generators:
            dx = DDElt({}, self.pmc_1, self.pmc_2rev)
            for y in self.generators:
                #Process the determinate chords.
                for (a,b) in self.DetChords+self.choice_chords:
                    #If a is compatible with x's left idempotents and y's left idempotents, and b is compatible with x's right idempotents and y's right idempotents, add corresponding elt to dx.
                    if l_idem_compat(self.pmc_1, a, x.idem_1) and l_idem_compat(self.pmc_2rev, self.opposite_strands(b), x.idem_2):
                        aalgelt = AlgElt(Strand_Diagram(self.pmc_1, a, x.idem_1))*AlgElt(Strand_Diagram(self.pmc_1, [], y.idem_1))
                        balgelt = AlgElt(Strand_Diagram(self.pmc_2rev, self.opposite_strands(b), x.idem_2))*AlgElt(Strand_Diagram(self.pmc_2rev, [], y.idem_2))
                        abalgelt = aalgelt**balgelt
                        if abalgelt:
                            dx = dx + DDElt({y:abalgelt}, self.pmc_1, self.pmc_2rev)
                #Now process the indeterminate chords -- write.
            diffs[x] = dx
        self.ddstr = TypeDDStr(self.pmc_1, self.pmc_2rev, self.generators, diffs)
        return self.ddstr

    def opposite_strands(self, strands):
        answer = list()
        for (x,y) in strands:
            answer.append((4*self.genus-y-1, 4*self.genus-x-1))
        return answer

#Method show() has been adjusted for overslides.
    def show(self):
        "Display graphically the pair (lchord, rchord). Intended for testing generate_U1() ... generate_U6()."
        pict = line([(0,-1), (0,4*self.genus)], rgbcolor = (0,0,0))
        pict = pict + line([(2,-1), (2,4*self.genus)], rgbcolor = (0,0,0))
        for i in range(4*self.genus):
            if i != self.b1:
                pict = pict + line([(0,i),(2,self.r(i))], rgbcolor = (1,0,0))
        if self.b1 < self.c1:
            pict = pict + line([(0,self.b1), (1,self.b1+1/2)], rgbcolor = (1,.25,.25)) + line([(1,self.b1p-1/2), (2,self.b1p)], rgbcolor = (1,.25,.25))
        if self.b1 > self.c1:
            pict = pict + line([(0,self.b1), (1,self.b1-1/2)], rgbcolor = (1,.25,.25)) + line([(1,self.b1p+1/2), (2,self.b1p)], rgbcolor = (1,.25,.25))
        #Show the matchings on the two sides:
        #left:
        for l in range(len(self.pmc_1.matching)):
            pict = pict + line([(-0.2, self.pmc_1.matching[l][0]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][0])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(-0.2, self.pmc_1.matching[l][1]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][1])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(-0.3-l/(8*self.genus), self.pmc_1.matching[l][0]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][1])], rgbcolor = (.2,.2,.2))
        #right:
        for l in range(len(self.pmc_2.matching)):
            pict = pict + line([(2.2, self.pmc_2.matching[l][0]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][0])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(2.2, self.pmc_2.matching[l][1]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][1])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(2.3+l/(8*self.genus), self.pmc_2.matching[l][0]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][1])], rgbcolor = (.2,.2,.2))
        pict.show(axes=False) #, xmin=-2,xmax=2, ymin=-.5, ymax=4*self.genus-.5)


    def display_chord(self, chords):
        "Display graphically the pair (lchord, rchord). Intended for testing generate_U1() ... generate_U6()."
        lchord = chords[0]
        rchord = chords[1]
        #First show the arcslide
        pict = line([(0,-1), (0,4*self.genus)], rgbcolor = (0,0,0))
        pict = pict + line([(2,-1), (2,4*self.genus)], rgbcolor = (0,0,0))
        for i in range(4*self.genus):
            if i != self.b1:
                pict = pict + line([(0,i),(2,self.r(i))], rgbcolor = (1,0,0))
        if self.b1 < self.c1:
            pict = pict + line([(0,self.b1), (1,self.b1+1/2)], rgbcolor = (1,.25,.25)) + line([(1,self.b1p-1/2), (2,self.b1p)], rgbcolor = (1,.25,.25))
        if self.b1 > self.c1:
            pict = pict + line([(0,self.b1), (1,self.b1-1/2)], rgbcolor = (1,.25,.25)) + line([(1,self.b1p+1/2), (2,self.b1p)], rgbcolor = (1,.25,.25))
        #light lines going farther out, for clarity.
        for x in range(4*self.genus):
            pict = pict + line([(-1,x),(0,x)], rgbcolor = (.8,.8,.8))
            pict = pict + line([(2,x),(3,x)], rgbcolor = (.8,.8,.8))
        #Show the matchings on the two sides:
        #left:
        for l in range(len(self.pmc_1.matching)):
            pict = pict + line([(-0.2, self.pmc_1.matching[l][0]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][0])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(-0.2, self.pmc_1.matching[l][1]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][1])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(-0.3-l/(8*self.genus), self.pmc_1.matching[l][0]), (-0.3-l/(8*self.genus), self.pmc_1.matching[l][1])], rgbcolor = (.2,.2,.2))
        #right:
        for l in range(len(self.pmc_2.matching)):
            pict = pict + line([(2.2, self.pmc_2.matching[l][0]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][0])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(2.2, self.pmc_2.matching[l][1]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][1])], rgbcolor = (.2,.2,.2))
            pict = pict + line([(2.3+l/(8*self.genus), self.pmc_2.matching[l][0]), (2.3+l/(8*self.genus), self.pmc_2.matching[l][1])], rgbcolor = (.2,.2,.2))
        #Now draw the chord on the two sides.
        for (x,y) in lchord:
            pict = pict + line([(-1,x),(-0.1, y)], rgbcolor = (0,0,1))
        for (x,y) in rchord:
            pict = pict + line([(2.1,x),(3, y)], rgbcolor = (0,0,1))
        pict.show(axes=False) #, xmin=-2,xmax=2, ymin=-.5, ymax=4*self.genus-.5)

class Arcslide():
    """Class for computing DD modules of arcslides. Contains an instance of either Overslide or Underslide, as appropriate.

    Input:
    pmc: The pointed matched circle you're arc-sliding.
    i: the moving endpoint of the arc-slide
    j: the endpoint i is sliding over

    Methods:
    dd_mod: generates and returns the type DD module for this underslide. The DD module is also stored; Calling dd_mod() only generates it the first time; afterwards, the existing dd structure is returned. (So, subsequent calls are quick.)
    show(): display this arc-slide graphically.

    Other useful data:
    self.pmc_2: the pointed matched circle resulting from the arc-slide.
    self.pmc_2rev: the orientation-reverse of self.pmc_2.

    Caveats:
    Currently restricts to the central SpinC structure, though this would be relatively easy to change.
    """
    def __init__(self, pmc, i, j=i+1):
        self.pmc_1 = pmc
        self.pmc_2 = pmc.arcslide(i,j)
        self.i=i
        self.j=j
        if self.pmc_1.is_overslide(i,j):
            self.arcslide = Overslide(self.pmc_1,i,j)
        if self.pmc_1.is_underslide(i,j):
            self.arcslide = Underslide(self.pmc_1,i,j)
        self.pmc_2rev = self.arcslide.pmc_2rev
        self.b1 = self.arcslide.b1
        self.b1p = self.arcslide.b1p
        self.c1 = self.arcslide.c1
        self.c1p = self.arcslide.c1p
        self.c2 = self.arcslide.c2
        self.c2p = self.arcslide.c2p

    def __repr__(self):
        if self.is_overslide():
            answer = "Overslide of "
        else:
            answer = "Underslide of "
        answer = answer + repr(self.i)+" over "+repr(self.j)+" of PMC "+repr(self.pmc_1)+"\n"
        return answer

    def dd_mod(self):
        return self.arcslide.dd_mod()

    def dd_mod_bar(self):
        return self.inverse().dd_mod().exchangeLR()
    
    def show(self):
        self.arcslide.show()

    def is_overslide(self):
        "Returns True if this is an overslide, False if an underslide."
        if self.pmc_1.is_overslide(self.i,self.j):
            return True
        return False

    def is_underslide(self):
        "Returns True if this is an underslide, False if an overslide."
        if self.pmc_1.is_overslide(self.i,self.j):
            return False
        return True

    def slide_type(self):
        "Returns 'Overslide' if this is an overslide, 'Underslide' if this is an underslide."
        if self.is_overslide():
            return "Overslide"
        return "Underslide"

    def inverse(self):
        "Returns the inverse arcslide to self."
        return Arcslide(self.pmc_2, self.b1p, self.c2p)



def linear_hb(g, print_status=False):
    "Returns the type D structure for the 'linear handlebody' of genus g, by doing a sequence of arc-slides on the 0-franed split handlebody."
    hb = dict()
    hb[0]=infty_type_D(g)
#    hb[0]=zero_type_D(g)
    slideseq = list()
    for i in range(g-1):
        slideseq.extend([(4*i+3,4*i+4),(4*i+6,4*i+7),(4*i+5,4*i+6)])
    arcslideseq = list()
    for i in range(len(slideseq)):
        if print_status:
            print "Performing slide number "+repr(i+1)+" of "+repr(3*(g-1))+"."
        hb[i+1] = Arcslide(hb[i].pmc,slideseq[i][0],slideseq[i][1]).dd_mod().mor_to_d(hb[i])
        hb[i+1].simplify()
        hb[i+1].shorten_names()
    return hb[3*(g-1)]

def linear_pmc(g):
    "Returns the linear pointed matched circle of genus g."
    matching = list()
    matching.append((0,2))
    for i in range(2*g-2):
        matching.append((2*i+1,2*i+4))
    matching.append((4*g-3,4*g-1))
    return PMC(g,matching)


class BraidGrp():
    """A class for doing computations of branched double covers of braids on num_strands strands. num_strands must be even.

    Methods: 
    ----generator(n): returns a list of Arcslides corresponding go sigma_n if n is positive, and sigma_{|n|}^{-1} if n is negative.
    ----hf_of_knot(braidword, print_status=False): returns the total rank of HF-hat of the branched double cover of the plat closure of braidword.  Here, braidword is a list of sigma_i's.  Passing print_status=True will result in verbose output, saying what it's currently computing.

    Example:
    sage: mygp = BraidGrp(6)
    sage: mygp.hf_of_knot(5*[1]+3*[3]+2*[-5])
    1
    #Computes HF-hat of the branched double cover of the (-2,3,5) pretzel knot (which gives the Poincare homology sphere.)    
    """
    def __init__(self, num_strands):
        self.num_strands = num_strands
        self.genus = (num_strands - 2)/2
        self.computed_gens = dict()
        self.pmc = linear_pmc(self.genus)
        self.handlebody = None

    def generator(self, n):
        """Return a list of Arcslides for the generator n of self. If n is positive, this is sigma_n.  If n is negative, this is sigma_n^{-1}."""
        if n in self.computed_gens.keys():
            return self.computed_gens[n]
        absn = abs(n)
        if absn == 1:
            abslidelist = [(1,0)]
        if 1<absn<self.num_strands-2:
            abslidelist = [(2*(absn-1),2*(absn-1)-1), (2*(absn-1),2*(absn-1)-1)]
        if absn == self.num_strands-2: #Just added this case; thanks Bohua.
            abslidelist = [(2*(absn-1),2*(absn-1)-1)]
        if absn == self.num_strands-1:
            temp = []
            for i in range(self.genus):
                temp = [(4*i+1,4*i)]+temp
            for i in range(2*self.genus-2):  
                temp.append((2*i+2,2*i+1))
            abslidelist = temp
        absarcslidelist = list()
        absarcslidelist.append(Arcslide(self.pmc,abslidelist[0][0],abslidelist[0][1]))
        for i in range(1,len(abslidelist)):
            absarcslidelist.append(Arcslide(absarcslidelist[i-1].pmc_2,abslidelist[i][0], abslidelist[i][1]))
        if n>0:
            self.computed_gens[n]=absarcslidelist
            return absarcslidelist
        if n<0:
            slidelist = list()
            for slide in absarcslidelist:
                slidelist = [slide.inverse()]+slidelist
            self.computed_gens[n]=slidelist
            return slidelist

    def hf_of_knot(self, braidword, print_status=False):
        """Returns the rank of HF-hat of the branched double cover of the plat closure of braidword.

        Braidword: a list of integers [a,b,c,d,...]. a stands for sigma_a if a is positive, and sigma_a^{-1} if a is negative.
        """
    #Keep track of which slides we've already used, so we don't re-compute their DD modules.
        if self.handlebody == None:
            if print_status:
                print "Generating handlebody for capping."
            self.handlebody = linear_hb(self.genus, print_status)
        current_hb = self.handlebody
        k=0
        for b in braidword:
            k=k+1
            i=0
            for s in self.generator(b):
                if print_status:
                    i=i+1
                    print "Performing handleslide "+repr(i)+" of "+repr(len(self.generator(b)))+" for generator "+repr(k)+" of "+repr(len(braidword))+ " (type s_"+repr(b)+")."
                current_hb = s.dd_mod().mor_to_d(current_hb)
                current_hb.simplify()
                current_hb.shorten_names()
        if print_status:
            print "Finishing by capping off."
        return self.handlebody.mor_to_d(current_hb).homology()






